#!/bin/bash

# This script will scan yearly folders for patmosx hdf files and call the Java 
# program to convert the data to NetCDF.  The NCO utility is then called to 
# convert the data to NetCDF-4 with internal compression and chunking.
# 
# To accomodate newer data being uploaded, the following conditions are used
# to filter the data.
# 1) If a NetCDF file already exists for a given HDF file and has
#    a newer last modified time, then the HDF file is ignored and 
#    deemed already processed.
#
# 2) If a HDF file has a last modified time within the last 5 
#    minutes, then ignore since it may be under modification/upload
#
# Author: Steve Ansari - Steve.Ansari@noaa.gov

datapath=/raid2/ftp/pub/upload/patmosx
tmppath=/raid2/ftp/pub/upload/tmp

mkdir $tmppath
rm $tmppath/*

for yyyy in 1981 1982 1983 1984 1985 1986 1987 1988 1989 1990 1991 1992 1993
 do

  #for x in $(ls $datapath/$yyyy/*.gz)
  # List directory and remove .gz extension from files
  for x in $(ls $datapath/$yyyy/*.gz | sed 's/\(.*\)\..*/\1/') 
   do
     filename=$(basename $x)

     #1. construct output netcdf filename
     #   01234567890123456789012345678901234567890
     #   patmosx_m02_des_2009_205_v05r02.hdf.gz
     #   patmosx_sw_m02_des_2009_205_v05r02.nc
     testNcFile=patmosx_sw_${filename:8:23}.nc
     hdfModTime=$(stat -c%Y $x.gz)

     #2. check if netcdf file exists
     if [ -e $datapath/NetCDF/$testNcFile ]
     then
        echo "exists!!!! $testNcFile"
        
        #3. if netcdf exists and is newer than hdf file, then ignore
        ncModTime=$(stat -c%Y $datapath/NetCDF/$testNcFile)
             
        if [ "$ncModTime" -gt "$hdfModTime" ]
        then
         echo "newer netcdf file found, file already processed... $x"
         continue
        fi
     fi

     #4. check if hdf file has been modified in last 5 minutes
     #   - if so, it may be under modification/upload
     currentTime=$(date +%s)
     timeDiff=$(( $currentTime-$hdfModTime ))
     if [ "$timeDiff" -lt 300 ]
     then
      echo "file may be under modification, skipping...  $x"
      continue
     fi

     #5. start processing
      
     echo "Decompressing: $x"
     gunzip -c $x.gz > $tmppath/$filename

     echo "Extracting: $tmppath/$filename"
     ./ncj-tools-0.2.0/convert-patmosx-to-nc $tmppath/$filename

     rm $tmppath/$filename

     for nc in $(ls $tmppath/*.nc)
      do
        echo "Converting $nc to NetCDF 4"
        ncks -Oh -4 -L 3 --cnk_dmn latitude,361 --cnk_dmn longitude,361 $nc $datapath/NetCDF/$(basename $nc) 
        rm $nc
      done
   done

 done
