! $Id: level2.f90,v 1.81 2010/09/23 15:38:26 wstraka Exp $
!----------------------------------------------------------------------------
!@***h* PATMOSX/level2.f90
! 
! NAME
!   level2.f90
! 
! LOCATION
!   /patmosx/src/level2.f90
! 
! PURPOSE
!   Routines for creating, writing and closing  pixel-level output files
! 
! DESCRIPTION
!   Routines for creating, writing and closing  pixel-level output files
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   May 2009
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   none
!
! EXTERNALS
!  use CONSTANTS
!  use AVHRR_PIXEL_COMMON
!  use HDF
!  use SCALING_PARAMETERS
!  use HDF_PARAMS
!  use OISST_ANALYSIS
!  
!  public:: DEFINE_HDF_FILE_STRUCTURES, &
!           WRITE_PIXEL_HDF_RECORDS, &
!           CLOSE_PIXEL_HDF_FILES
!  private::DEFINE_PIXEL_2D_SDS
!
!@*****
!------------------------------------------------------------------------
!
! MODULE name: Level2_routines
! 
! Routines for creating, writing and closing  pixel-level output files
!
! Author: Andrew Heidinger, NOAA/NESDIS
!
! History:
!          5/09 - Created
!----------------------------------------------------------------------------
module LEVEL2_ROUTINES
   use CONSTANTS
   use AVHRR_PIXEL_COMMON
   use HDF
   use SCALING_PARAMETERS
   use HDF_PARAMS
   use OISST_ANALYSIS
   implicit none
   private
   public:: DEFINE_HDF_FILE_STRUCTURES, &
            WRITE_PIXEL_HDF_RECORDS, &
            CLOSE_PIXEL_HDF_FILES
   private::DEFINE_PIXEL_2D_SDS

!----------------------------------------------------------------------
! the following variables taken from process_avhrr_clavr
!----------------------------------------------------------------------
!--- hdf specific variables
 integer(kind=int4),private:: dim_id

 !--- cmr indices
 integer, parameter, private:: num_cmr_sds = 7
 integer, private, save:: sd_id_cmr
 integer(kind=int4), dimension(num_cmr_sds), save, private:: sds_id_cmr

 !--- nav indices
 integer, parameter, private:: num_nav_sds = 6
 integer, private, save:: sd_id_nav
 integer(kind=int4), dimension(num_nav_sds), save, private:: sds_id_nav

 !--- sst indices
 integer, parameter, private:: num_sst_sds = 7
 integer, private, save:: sd_id_sst
 integer(kind=int4), dimension(num_sst_sds), save, private:: sds_id_sst

 !--- cld indices
 integer, parameter, private:: num_cld_sds = 15
 integer, private, save:: sd_id_cld
 integer(kind=int4), dimension(num_cld_sds), save, private:: sds_id_cld

 !--- ash indices
 integer, parameter, private:: num_ash_sds_diag = 23
 integer, parameter, private:: num_ash_sds_oper = 7
 integer, private, save:: sd_id_ash
 integer(kind=int4), dimension(num_ash_sds_diag), save, private:: sds_id_ash

 !--- level2 indices
 integer, parameter, private:: num_level2_sds = 41
 integer, private, save:: sd_id_level2
 integer(kind=int4), dimension(num_level2_sds), save, private:: sds_id_level2

 !--- obs indices
 integer, parameter, private:: num_obs_sds = 9
 integer, private, save:: sd_id_obs
 integer(kind=int4), dimension(num_obs_sds), save, private:: sds_id_obs

 !--- geo indices
 integer, parameter, private:: num_geo_sds = 11
 integer, private, save:: sd_id_geo
 integer(kind=int4), dimension(num_geo_sds), save, private:: sds_id_geo

 !--- rtm indices
 integer, parameter, private:: num_rtm_sds = 22
 integer, private, save:: sd_id_rtm
 integer(kind=int4), dimension(num_rtm_sds), save, private:: sds_id_rtm

 !--- compression variables
 integer(kind=int4), dimension(2),private, save::  comp_prm
 integer(kind=int4), private, save::  comp_type

 integer(kind=int4),parameter,private:: sds_rank = 2
 integer(kind=int4),dimension(sds_rank),private:: sds_dims_2d
 integer(kind=int4),dimension(sds_rank),private:: anchor_sds_dims_2d
 integer(kind=int4),dimension(sds_rank),private:: sds_start
 integer(kind=int4),dimension(sds_rank),private:: sds_stride
 integer(kind=int4),dimension(sds_rank),private:: sds_edge
 integer(kind=int4),dimension(sds_rank),private:: anchor_sds_edge
 integer(kind=int4),dimension(sds_rank),private:: sds_chunk_size
 integer(kind=int4),dimension(sds_rank),private:: anchor_sds_chunk_size
 integer(kind=int4),dimension(1),private:: sds_dims_1d

! Arrays used for slab reading from cmr file.
 integer(kind=int2), dimension(1),private:: cmr_scan_number
 integer(kind=int1), dimension(:), allocatable,private:: cmr_one_byte

 character(len=11), private, parameter:: MOD_PROMPT = "LEVEL2:"

!--- set sds order for level2 files
 integer, private, parameter::    &
     sds_num_level2_scanline = 1, &
     sds_num_level2_time = 2, &
     sds_num_level2_bad_scan = 3, &
     sds_num_level2_asc_flag = 4, &
     sds_num_level2_meta_data = 5, &
     sds_num_level2_lat = 6, &
     sds_num_level2_lon = 7, &
     sds_num_level2_zen = 8, &
     sds_num_level2_solzen = 9, &
     sds_num_level2_relaz = 10 , &
     sds_num_level2_solaz = 11 , &
     sds_num_level2_land = 12 , &
     sds_num_level2_ch1 = 13 , &
     sds_num_level2_ch2 = 14 , &
     sds_num_level2_ch3a = 15 , &
     sds_num_level2_ch3b = 16 , &
     sds_num_level2_ch3b4 = 17 , &
     sds_num_level2_ch4 = 18 , &
     sds_num_level2_ch45 = 19 , &
     sds_num_level2_ch1_std = 20 , &
     sds_num_level2_ch4_std = 21 , &
     sds_num_level2_cldprob = 22, &
     sds_num_level2_ctype = 23 , &
     sds_num_level2_ctp = 24, &
     sds_num_level2_ctt = 25, &
     sds_num_level2_ec = 26, &
     sds_num_level2_beta = 27, &
     sds_num_level2_cod = 28, &
     sds_num_level2_ceps = 29, &
     sds_num_level2_cldalb = 30, &
     sds_num_level2_cldtrn = 31, &
     sds_num_level2_cldfrac = 32, &
     sds_num_level2_cldfracuncer = 33, &
     sds_num_level2_etrop = 34, &
     sds_num_level2_aot1 = 35, &
     sds_num_level2_olr = 36, &
     sds_num_level2_ndvi = 37, &
     sds_num_level2_tsfc = 38, &
     sds_num_level2_rsr = 39, &
     sds_num_level2_qf1 = 40, &
     sds_num_level2_qf2 = 41

  integer, parameter, private:: num_sds_ncdc_level2 = sds_num_level2_cldprob
contains

!====================================================================
! SUBROUTINE Name: DEFINE_HDF_FILE_STRUCTURES
!
! Function:
!   Determines the structure of the level 2 files.
!
! Description:
!   This subroutine, given the inputs, determins and opens/creates the apppropriate
!       Level 2 (pixel level output) for various types of files. In addition
!       the HDF global attributes are put into the various HDF files 
!
!====================================================================

subroutine DEFINE_HDF_FILE_STRUCTURES(num_scans, &
   dir_cmr, &
   dir_nav, &
   dir_sst, &
   dir_cld, &
   dir_obs, &
   dir_geo, &
   dir_rtm, &
   dir_ash, &
   dir_level2, &
   file_1b, &
   cmr_file_flag, &
   nav_file_flag, &
   sst_file_flag, &
   cld_file_flag, &
   obs_file_flag, &
   geo_file_flag, &
   rtm_file_flag, &
   ash_file_flag, &
   level2_file_flag, &
   c1,c2,a1_3b, &
   a2_3b, &
   nu_3b, &
   a1_4, &
   a2_4, &
   nu_4, &
   a1_5, &
   a2_5, &
   nu_5, &
   solar_3b_nu, &
   sun_earth_distance, &
   therm_cal_1b,  &
   ref_cal_1b, &
   nav_flag, &
   use_sst_anal, &
   sst_anal_opt, &
   cloud_type_ir_flag, &
   modis_clr_alb_flag, &
   nwp_flag, &
   ch1_gain_low, &
   ch1_gain_high, &
   ch1_switch_count, &
   ch1_dark_count,&
   ch2_gain_low, &
   ch2_gain_high, &
   ch2_switch_count, &
   ch2_dark_count, &
   ch3a_gain_low, &
   ch3a_gain_high, &
   ch3a_switch_count, &
   ch3a_dark_count,&
   start_year, &
   end_year, &
   start_day, &
   end_day, &
   start_time, &
   end_time)

 integer(kind=int4), intent(in):: num_scans
 character(len=*), intent(in):: dir_cmr
 character(len=*), intent(in):: dir_nav
 character(len=*), intent(in):: dir_sst
 character(len=*), intent(in):: dir_cld
 character(len=*), intent(in):: dir_obs
 character(len=*), intent(in):: dir_geo
 character(len=*), intent(in):: dir_rtm
 character(len=*), intent(in):: dir_ash
 character(len=*), intent(in):: dir_level2
 character(len=*), intent(in):: file_1b
 integer, intent(in):: cmr_file_flag
 integer, intent(in):: nav_file_flag
 integer, intent(in):: obs_file_flag
 integer, intent(in):: geo_file_flag
 integer, intent(in):: rtm_file_flag
 integer, intent(in):: ash_file_flag
 integer, intent(in):: sst_file_flag
 integer, intent(in):: cld_file_flag
 integer, intent(in):: level2_file_flag
 integer, intent(in):: sst_anal_opt
 integer(kind=int4), intent(in) :: modis_clr_alb_flag
 integer(kind=int4), intent(in) :: nwp_flag
 integer, intent(in):: therm_cal_1b
 integer, intent(in):: nav_flag
 integer, intent(in):: use_sst_anal
 integer, intent(in):: cloud_type_ir_flag
 integer, intent(in):: ref_cal_1b
 real(kind=real4), intent(in):: c1,c2,a1_3b,a2_3b,nu_3b,a1_4,a2_4,nu_4,a1_5, &
                                a2_5,nu_5,solar_3b_nu,sun_earth_distance, &
                                ch1_gain_low,ch1_gain_high,ch1_switch_count, &
                                ch1_dark_count, ch2_gain_low,ch2_gain_high, &
                                ch2_switch_count,ch2_dark_count, ch3a_gain_low,&
                                ch3a_gain_high,ch3a_switch_count, &
                                ch3a_dark_count
 integer(kind=int4), intent(in):: start_time
 integer(kind=int4), intent(in):: end_time
 integer(kind=int2), intent(in):: start_year
 integer(kind=int2), intent(in):: end_year
 integer(kind=int2), intent(in):: start_day
 integer(kind=int2), intent(in):: end_day
 character(len=128):: file_cmr
 character(len=128):: file_nav
 character(len=128):: file_sst
 character(len=128):: file_cld
 character(len=128):: file_obs
 character(len=128):: file_geo
 character(len=128):: file_rtm
 character(len=128):: file_ash
 character(len=128):: file_level2

 character(len=128):: long_name_temp
 character(len=128):: standard_name_temp

 integer(kind=int4):: blank_int4
 character(len=1):: blank_char
 real:: blank_real4
 integer:: istatus_sum
 integer:: istatus

! HDF function declarations
 integer:: sfstart
 integer:: sfcreate
 integer:: sfscatt
 integer:: sfsnatt
 integer:: erstat

!--- begin executable code
     blank_int4 = 0
     blank_real4 = 0.0
     blank_char = " "

!-------------------------------------------------------------
! define chunking here
!-------------------------------------------------------------
     sds_chunk_size(1) = num_pix
     sds_chunk_size(2) = num_scans_per_segment

     anchor_sds_chunk_size(1) = num_anchors
     anchor_sds_chunk_size(2) = num_scans_per_segment

     !-- dimension of 2d variables
     sds_dims_2d(1) = num_pix
     sds_dims_2d(2) = num_scans

     !-- dimension of 2d variables for anchors
     anchor_sds_dims_2d(1) = num_anchors
     anchor_sds_dims_2d(2) = num_scans

!-- dimension of 1d variables
     sds_dims_1d(1) =  num_scans

!--- if subsetting pixel hdf, set scan dimension to 0 (sd_unlimited)
!--- note, this turns off compression
     if (subset_pixel_hdf_flag == sym%YES) then
          sds_dims_2d(2) = 0   !SD_UNLIMITED
!         sds_dims_1d(1) = 0   !- this is does not seem to work
     endif

!-------------------------------------------------------------
! define compression here
!-------------------------------------------------------------
 comp_type = 0                  !no compression
 comp_prm(1) = 0
 comp_prm(2) = 0

 if (data_comp_flag == 1) then  !gzip compression
   comp_type = 4
   comp_prm(1) = 6
   comp_prm(2) = 0
 endif

 if (data_comp_flag == 2) then  !szip compression
   comp_type = 5
   comp_prm(1) = 32
   comp_prm(2) = 2
 endif

!---------------------------------------------------------
!-- define sst file structure
!---------------------------------------------------------
  if (sst_file_flag == sym%YES) then

     file_sst = trim(file_1b)//".sst.hdf"
     print *, EXE_PROMPT, MOD_PROMPT, "creating SST file ", trim(file_sst)
     sd_id_sst = sfstart(trim(dir_sst)//trim(file_sst),DFACC_CREATE)
     if (sd_id_sst .lt. 0) then
       erstat = 68
       print *, EXE_PROMPT, MOD_PROMPT, "SST file creation failed. Exiting..."
       stop 68
     endif

     !--- write attributes
     call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_sst,"PIXEL",file_sst,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           blank_int4,blank_int4,blank_char,blank_real4, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, modis_clr_alb_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a1_3b, a2_3b, nu_3b, &
                           a1_4, a2_4, nu_4, a1_5, a2_5, nu_5, &
                           solar_3b_nu,timerr_seconds,sc_id_new)

     !-- reset status flag for error checking
     istatus_sum = 0

     !-- scan line variables
     sds_id_sst(1) = sfcreate(sd_id_sst,"scan_line_number",DFNT_INT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_sst(1), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_sst(1), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_sst(1), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     sds_id_sst(2) = sfcreate(sd_id_sst,"scan_line_time",DFNT_FLOAT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_sst(2), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_sst(2), "UNITS", DFNT_CHAR8, 5, "hours") + istatus_sum
     istatus_sum = sfsnatt(sds_id_sst(2), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     !--- sst
     call DEFINE_PIXEL_2D_SDS(sds_id_sst(3),sd_id_sst,sds_dims_2d,sds_chunk_size, &
                              "pixel_sst", &
                              "pixel_sst", &
                              "pixel_sst", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_tsfc, max_tsfc, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- packed cld mask
     call DEFINE_PIXEL_2D_SDS(sds_id_sst(4),sd_id_sst,sds_dims_2d,sds_chunk_size, &
                              "packed_cld_mask", &
                              "packed_cld_mask", &
                              "packed_cld_mask", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- sst analysis
     call DEFINE_PIXEL_2D_SDS(sds_id_sst(5),sd_id_sst,sds_dims_2d,sds_chunk_size, &
                              "pixel_sst_background", &
                              "pixel_sst_background", &
                              "pixel_sst_background", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_sst, max_sst, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- sst quality flag
     call DEFINE_PIXEL_2D_SDS(sds_id_sst(6),sd_id_sst,sds_dims_2d,sds_chunk_size, &
                              "sst_quality_flag", &
                              "sst_quality_flag", &
                              "sst_quality_flag", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- sst masked
     call DEFINE_PIXEL_2D_SDS(sds_id_sst(7),sd_id_sst,sds_dims_2d,sds_chunk_size, &
                              "pixel_sst_masked", &
                              "pixel_sst_masked", &
                              "pixel_sst_masked", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_sst, max_sst, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- check for and report errors
     if (istatus_sum /= 0) then
       print *, EXE_PROMPT, MOD_PROMPT, "Error defining sds in sst hdf file"
     endif

  endif

!---------------------------------------------------------
!-- define cmr file structure
!---------------------------------------------------------
   if (cmr_file_flag == sym%YES) then
     file_cmr = trim(file_1b)//".cmr.hdf"
     print *, EXE_PROMPT, MOD_PROMPT, "creating CMR file ", trim(file_cmr)
     sd_id_cmr = sfstart(trim(dir_cmr)//trim(file_cmr),DFACC_CREATE)
     if (sd_id_cmr .lt. 0) then
           print *, EXE_PROMPT, MOD_PROMPT, "Creation of cmr file failed. Exiting..."
           erstat = 69
           stop 69
    endif
                                                                                                                                            
     !--- write global attributes
     call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_cmr,"PIXEL",file_cmr,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           blank_int4,blank_int4,blank_char,blank_real4, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, modis_clr_alb_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a1_3b, a2_3b, nu_3b, &
                           a1_4, a2_4, nu_4, a1_5, a2_5, nu_5, &
                           solar_3b_nu,timerr_seconds,sc_id_new)

     !--- reset error status flag
     istatus_sum = 0

     !--- scan line 
     sds_id_cmr(1) = sfcreate(sd_id_cmr,"scan_line_number",DFNT_INT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_cmr(1), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_cmr(1), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_cmr(1), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     !--- scan line time
     sds_id_cmr(2) = sfcreate(sd_id_cmr,"scan_line_time",DFNT_FLOAT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_cmr(2), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_cmr(2), "UNITS", DFNT_CHAR8, 5, "hours") + istatus_sum
     istatus_sum = sfsnatt(sds_id_cmr(2), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     !--- cld mask
     call DEFINE_PIXEL_2D_SDS(sds_id_cmr(3),sd_id_cmr,sds_dims_2d,sds_chunk_size, &
                              "clavr-x_cld_mask_results", &
                              "clavr-x_cld_mask_results", &
                              "clavr-x_cld_mask_results", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- cld mask tests
     call DEFINE_PIXEL_2D_SDS(sds_id_cmr(4),sd_id_cmr,sds_dims_2d,sds_chunk_size, &
                              "cld_mask_test_results_1", &
                              "cld_mask_test_results_1", &
                              "cld_mask_test_results_1", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     call DEFINE_PIXEL_2D_SDS(sds_id_cmr(5),sd_id_cmr, &
                              sds_dims_2d, sds_chunk_size,&
                              "cld_mask_test_results_2", &
                              "cld_mask_test_results_2", &
                              "cld_mask_test_results_2", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     call DEFINE_PIXEL_2D_SDS(sds_id_cmr(6),sd_id_cmr,sds_dims_2d,sds_chunk_size, &
                              "cld_mask_test_results_3", &
                              "cld_mask_test_results_3", &
                              "cld_mask_test_results_3", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     call DEFINE_PIXEL_2D_SDS(sds_id_cmr(7),sd_id_cmr,sds_dims_2d,sds_chunk_size, &
                              "cld_mask_test_results_4", &
                              "cld_mask_test_results_4", &
                              "cld_mask_test_results_4", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- check for and report errors
     if (istatus_sum /= 0) then
       print *, EXE_PROMPT, MOD_PROMPT, "Error defining sds in cmr hdf file ", istatus
       stop
     endif

  endif

!---------------------------------------------------------
!-- define navigation file structure
!---------------------------------------------------------
  if (nav_file_flag == sym%YES) then
     file_nav = trim(file_1b)//".nav.hdf"
     print *, EXE_PROMPT, MOD_PROMPT, "creating NAV file ", trim(file_nav)
     sd_id_nav = sfstart(trim(dir_nav)//trim(file_nav),DFACC_CREATE)
     if (sd_id_nav .lt. 0) then
        print *, EXE_PROMPT, MOD_PROMPT, "Creation of navigation file failed. Exiting..."
        erstat = 71
        stop 71
     endif

!--- write global attributes
 call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_nav,"PIXEL",file_nav,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           blank_int4,blank_int4,blank_char,blank_real4, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, modis_clr_alb_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a1_3b, a2_3b, nu_3b, &
                           a1_4, a2_4, nu_4, a1_5, a2_5, nu_5, &
                           solar_3b_nu,timerr_seconds,sc_id_new)

     !-- reset status flag for error checking
     istatus_sum = 0

!--- scan number
     sds_id_nav(1) = sfcreate(sd_id_nav,"scan_line_number",DFNT_INT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_nav(1), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_nav(1), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_nav(1), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

!--- scan time
     sds_id_nav(2) = sfcreate(sd_id_nav,"scan_line_time",DFNT_FLOAT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_nav(2), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_nav(2), "UNITS", DFNT_CHAR8, 5, "hours") + istatus_sum
     istatus_sum = sfsnatt(sds_id_nav(2), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

!--- asc des flag
     sds_id_nav(3) = sfcreate(sd_id_nav,"asc_des_flag",DFNT_INT8,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_nav(3), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_nav(3), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum

!--- bad scan flag
     sds_id_nav(4) = sfcreate(sd_id_nav,"bad_scan_line_flag",DFNT_INT8,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_nav(4), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_nav(4), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum

!--- latitude
     call DEFINE_PIXEL_2D_SDS(sds_id_nav(5),sd_id_nav,sds_dims_2d,sds_chunk_size, &
                              "pixel_latitude", &
                              "pixel_latitude", &
                              "pixel_latitude", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_lat, max_lat, "degrees", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- longitude
     call DEFINE_PIXEL_2D_SDS(sds_id_nav(6),sd_id_nav,sds_dims_2d,sds_chunk_size, &
                              "pixel_longitude", &
                              "pixel_longitude", &
                              "pixel_longitude", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_lon, max_lon, "degrees", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- check for and report errors
     if (istatus_sum /= 0) then
       print *, EXE_PROMPT, MOD_PROMPT, "Error defining sds in nav hdf file ", istatus
       stop
     endif

  endif

!---------------------------------------------------------
!-- define cld product file structure
!---------------------------------------------------------
  if (cld_file_flag == sym%YES) then
     file_cld = trim(file_1b)//".cld.hdf"
     print *, EXE_PROMPT, MOD_PROMPT, "creating CLD file ", trim(file_cld)
     sd_id_cld = sfstart(trim(dir_cld)//trim(file_cld),DFACC_CREATE)
     if (sd_id_cld < 0) then
        print *, EXE_PROMPT, MOD_PROMPT, "Creation of cld product file failed. Exiting..."
        erstat = 72
        stop 72
     endif

!--- write global attributes
 call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_cld,"PIXEL",file_cld,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           blank_int4,blank_int4,blank_char,blank_real4, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, modis_clr_alb_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a1_3b, a2_3b, nu_3b, &
                           a1_4, a2_4, nu_4, a1_5, a2_5, nu_5, &
                           solar_3b_nu,timerr_seconds,sc_id_new)

!--- reset error counter
     istatus_sum = 0

!--- scan line
     sds_id_cld(1) = sfcreate(sd_id_cld,"scan_line_number",DFNT_INT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_cld(1), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_cld(1), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_cld(1), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

!--- scan time
     sds_id_cld(2) = sfcreate(sd_id_cld,"scan_line_time",DFNT_FLOAT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_cld(2), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_cld(2), "UNITS", DFNT_CHAR8, 5, "hours") + istatus_sum
     istatus_sum = sfsnatt(sds_id_cld(2), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

!--- cloud mask
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(3),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "cloud_mask", &
                              "cloud_mask", &
                              "cloud_mask", &
                              DFNT_INT8, sym%NO_SCALING, 0.0, 0.0, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- cloud type
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(4),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "cloud_type", &
                              "cloud_type", &
                              "cloud_type", &
                              DFNT_INT8, sym%NO_SCALING, 0.0, 0.0, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- cloud temperature from the split window approach
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(5),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "cld_temp_sw", &
                              "cld_temp_sw", &
                              "cld_temp_sw", &
                              DFNT_INT8, sym%LINEAR_SCALING, min_tc, max_tc, &
                              "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- cloud emissivity from the split window approach
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(6),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "cld_emiss_sw", &
                              "cld_emiss_sw", &
                              "cld_emiss_sw", &
                              DFNT_INT8, sym%LINEAR_SCALING, min_ec, max_ec, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- beta from the split-window approach
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(7),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "cld_beta_sw", &
                              "cld_beta_sw", &
                              "cld_beta_sw", &
                              DFNT_INT8, sym%LINEAR_SCALING, min_beta, max_beta, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- cloud height from the split-window approach
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(8),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "cld_height_sw", &
                              "cld_height_sw", &
                              "cld_height_sw", &
                              DFNT_INT8, sym%LINEAR_SCALING, min_zc, max_zc, &
                              "km", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- packed cloud quality flags for split window algorithm
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(9),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "sw_packed_cld_quality_flags", &
                              "sw_packed_cld_quality_flags", &
                              "sw_packed_cld_quality_flags", &
                              DFNT_INT8, sym%NO_SCALING, 0.0, 0.0, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- cloud optical depth from the vis/nir approach
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(10),sd_id_cld,sds_dims_2d, sds_chunk_size,&
                              "cld_opd_vis", &
                              "cld_opd_vis", &
                              "cld_opd_vis", &
                              DFNT_INT16, sym%SQUARE_ROOT_SCALING, min_tau, max_tau, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- cloud effective particle radius from the vis/nir approach
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(11),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "cld_reff_vis", &
                              "cld_reff_vis", &
                              "cld_reff_vis", &
                              DFNT_INT16, sym%SQUARE_ROOT_SCALING, min_re, max_re, &
                              "micron", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- packed cloud quality flags for vis/nir algorithm
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(12),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "vis_packed_cld_quality_flags", &
                              "vis_packed_cld_quality_flags", &
                              "vis_packed_cld_quality_flags", &
                              DFNT_INT8, sym%NO_SCALING, 0.0, 0.0, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- infrared cloud type
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(13),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "cloud_type_ir", &
                              "cloud_type_ir", &
                              "cloud_type_ir", &
                              DFNT_INT8, sym%NO_SCALING, 0.0, 0.0, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- cloud pressure from the split-window approach
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(14),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "cld_press_sw", &
                              "cld_press_sw", &
                              "cld_press_sw", &
                              DFNT_INT8, sym%LINEAR_SCALING, min_pc, max_pc, &
                              "hPa", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- cloud temperature error from the split-window approach
     call DEFINE_PIXEL_2D_SDS(sds_id_cld(15),sd_id_cld,sds_dims_2d,sds_chunk_size, &
                              "cld_temp_sw_error", &
                              "cld_temp_sw_error", &
                              "cld_temp_sw_error", &
                              DFNT_INT8, sym%LINEAR_SCALING, min_tc_error, max_tc_error, &
                              "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus


!--- check and report errors
     if (istatus_sum /= 0) then
       print *, EXE_PROMPT, MOD_PROMPT, "Error in defining cld sds ", istatus_sum
     endif 

  endif

!---------------------------------------------------------
!-- define obs file structure
!---------------------------------------------------------
  if (obs_file_flag == sym%YES) then
     file_obs = trim(file_1b)//".obs.hdf"
     print *, EXE_PROMPT, MOD_PROMPT, "creating OBS file ", trim(file_obs)
     sd_id_obs = sfstart(trim(dir_obs)//trim(file_obs),DFACC_CREATE)
     if (sd_id_obs < 0) then
        print *, EXE_PROMPT, MOD_PROMPT, "Creation of OBS product file failed. Exiting..."
        erstat = 73
        stop 73
     endif

!--- write global attributes
 call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_obs,"PIXEL",file_obs,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           blank_int4,blank_int4,blank_char,blank_real4, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, modis_clr_alb_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a1_3b, a2_3b, nu_3b, &
                           a1_4, a2_4, nu_4, a1_5, a2_5, nu_5, &
                           solar_3b_nu,timerr_seconds,sc_id_new)

!-- reset status flag for error checking
     istatus_sum = 0

!-- scan line variables
     sds_id_obs(1) = sfcreate(sd_id_obs,"scan_line_number",DFNT_INT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_obs(1), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_obs(1), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_obs(1), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     sds_id_obs(2) = sfcreate(sd_id_obs,"scan_line_time",DFNT_FLOAT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_obs(2), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_obs(2), "UNITS", DFNT_CHAR8, 5, "hours") + istatus_sum
     istatus_sum = sfsnatt(sds_id_obs(2), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     sds_id_obs(3) = sfcreate(sd_id_obs,"bad_scan_line_flag",DFNT_INT8,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_obs(3), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_obs(3), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_obs(3), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

!--- ch1 reflectance
     call DEFINE_PIXEL_2D_SDS(sds_id_obs(4),sd_id_obs,sds_dims_2d,sds_chunk_size, &
                              "ch1_reflectance", &
                              "ch1_reflectance", &
                              "ch1_reflectance", &
                              DFNT_INT16, sym%SQUARE_ROOT_SCALING, &
                              min_alb1, max_alb1, "%", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- ch2 reflectance
     call DEFINE_PIXEL_2D_SDS(sds_id_obs(5),sd_id_obs,sds_dims_2d,sds_chunk_size, &
                              "ch2_reflectance", &
                              "ch2_reflectance", &
                              "ch2_reflectance", &
                              DFNT_INT8, sym%SQUARE_ROOT_SCALING, &
                              min_alb2, max_alb2, "%", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!-- ch3ab reflectance
     call DEFINE_PIXEL_2D_SDS(sds_id_obs(6),sd_id_obs,sds_dims_2d,sds_chunk_size, &
                              "ch3ab_reflectance", &
                              "ch3ab_reflectance", &
                              "ch3ab_reflectance", &
                              DFNT_INT8, sym%SQUARE_ROOT_SCALING, &
                              min_alb3b, max_alb3a, "%", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- ch3b temperature
     call DEFINE_PIXEL_2D_SDS(sds_id_obs(7),sd_id_obs,sds_dims_2d,sds_chunk_size, &
                              "ch3b_temperature", &
                              "ch3b_temperature", &
                              "ch3b_temperature", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_t3b, max_t3b, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- ch4 temperature
     call DEFINE_PIXEL_2D_SDS(sds_id_obs(8),sd_id_obs,sds_dims_2d,sds_chunk_size, &
                              "ch4_temperature", &
                              "ch4_temperature", &
                              "ch4_temperature", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_t4, max_t4, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- ch5 temperature
     call DEFINE_PIXEL_2D_SDS(sds_id_obs(9),sd_id_obs,sds_dims_2d,sds_chunk_size, &
                              "ch5_temperature", &
                              "ch5_temperature", &
                              "ch5_temperature", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_t5, max_t5, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- check for and report errors
     if (istatus_sum /= 0) then
       print *, EXE_PROMPT, MOD_PROMPT, "Error defining sds in obs hdf file"
     endif

  endif

!---------------------------------------------------------
!-- define geo(graphic) file structure
!---------------------------------------------------------
 if (geo_file_flag == sym%YES) then
     file_geo = trim(file_1b)//".geo.hdf"
     print *, EXE_PROMPT, MOD_PROMPT, "creating GEO file ", trim(file_geo)
     sd_id_geo = sfstart(trim(dir_geo)//trim(file_geo),DFACC_CREATE)
     if (sd_id_geo < 0) then
        print *, EXE_PROMPT, MOD_PROMPT, "Creation of GEO product file failed. Exiting..."
        erstat = 74
        stop 74
     endif

!--- write global attributes
 call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_geo,"PIXEL",file_geo,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           blank_int4,blank_int4,blank_char,blank_real4, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, modis_clr_alb_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a1_3b, a2_3b, nu_3b, &
                           a1_4, a2_4, nu_4, a1_5, a2_5, nu_5, &
                           solar_3b_nu,timerr_seconds,sc_id_new)

!--- reset counter
     istatus_sum = 0

!-- scan line
     sds_id_geo(1) = sfcreate(sd_id_geo,"scan_line_number",DFNT_INT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_geo(1), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_geo(1), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_geo(1), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

!--- scan time
     sds_id_geo(2) = sfcreate(sd_id_geo,"scan_line_time",DFNT_FLOAT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_geo(2), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_geo(2), "UNITS", DFNT_CHAR8, 5, "hours") + istatus_sum
     istatus_sum = sfsnatt(sds_id_geo(2), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

!-- pixel variables
     call DEFINE_PIXEL_2D_SDS(sds_id_geo(3),sd_id_geo,sds_dims_2d,sds_chunk_size, &
                              "surface_type", &
                              "surface_type", &
                              "surface_type", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- surface elevation
     call DEFINE_PIXEL_2D_SDS(sds_id_geo(4),sd_id_geo,sds_dims_2d,sds_chunk_size, &
                              "surface_elevation", &
                              "surface_elevation", &
                              "surface_elevation", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_z, max_z, "meters", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- sensor zenith angle
     call DEFINE_PIXEL_2D_SDS(sds_id_geo(5),sd_id_geo,sds_dims_2d,sds_chunk_size, &
                              "sensor_zenith", &
                              "sensor_zenith", &
                              "sensor_zenith", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_ang, max_ang, "degrees", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- solar zenith angle
     call DEFINE_PIXEL_2D_SDS(sds_id_geo(6),sd_id_geo,sds_dims_2d,sds_chunk_size, &
                              "solar_zenith", &
                              "solar_zenith", &
                              "solar_zenith_angle", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_ang, max_ang, "degrees", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- relative azimuth
     call DEFINE_PIXEL_2D_SDS(sds_id_geo(7),sd_id_geo,sds_dims_2d,sds_chunk_size, &
                              "relative_azimuth", &
                              "relative_azimuth", &
                              "relative_azimuth_angle", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_ang, max_ang, "degrees", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- coast
     call DEFINE_PIXEL_2D_SDS(sds_id_geo(8),sd_id_geo,sds_dims_2d,sds_chunk_size, &
                              "coast", &
                              "coast", &
                              "coast", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "km", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- solar azimuth
     call DEFINE_PIXEL_2D_SDS(sds_id_geo(9),sd_id_geo,sds_dims_2d,sds_chunk_size, &
                              "solar_azimuth", &
                              "solar_azimuth", &
                              "solar_azimuth", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_solaz, max_solaz, "degrees", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- land class
     call DEFINE_PIXEL_2D_SDS(sds_id_geo(10),sd_id_geo,sds_dims_2d,sds_chunk_size, &
                              "land_classification", &
                              "land_classification", &
                              "land_classification", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "unknown", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- snow
     call DEFINE_PIXEL_2D_SDS(sds_id_geo(11),sd_id_geo,sds_dims_2d,sds_chunk_size, &
                              "snow_ice_flag", &
                              "snow_ice_flag", &
                              "snow_ice_flag", &
                              DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "unknown", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!--- check for and report errors
     if (istatus_sum /= 0) then
       print *, EXE_PROMPT, MOD_PROMPT, "Error defining sds in geo hdf file"
     endif

endif

!---------------------------------------------------------
!-- define rtm file structure
!---------------------------------------------------------
 if (rtm_file_flag == sym%YES) then

     file_rtm = trim(file_1b)//".rtm.hdf"
     print *, EXE_PROMPT, MOD_PROMPT, "creating RTM file ", trim(file_rtm)

     sd_id_rtm = sfstart(trim(dir_rtm)//trim(file_rtm),DFACC_CREATE)

     if (sd_id_rtm < 0) then
        print *, EXE_PROMPT, MOD_PROMPT, "Creation of RTM product file failed. Exiting..."
        erstat = 74
        stop 74
     endif

     !--- write global attributes
     call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_rtm,"PIXEL",file_rtm,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           blank_int4,blank_int4,blank_char,blank_real4, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, modis_clr_alb_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a1_3b, a2_3b, nu_3b, &
                           a1_4, a2_4, nu_4, a1_5, a2_5, nu_5, &
                           solar_3b_nu,timerr_seconds,sc_id_new)

     !-- reset status flag for error checking
     istatus_sum = 0

     !-- scan line
     sds_id_rtm(1) = sfcreate(sd_id_rtm,"scan_line_number",DFNT_INT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_rtm(1), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_rtm(1), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_rtm(1), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     !--- scan time
     sds_id_rtm(2) = sfcreate(sd_id_rtm,"scan_line_time",DFNT_FLOAT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_rtm(2), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_rtm(2), "UNITS", DFNT_CHAR8, 5, "hours") + istatus_sum
     istatus_sum = sfsnatt(sds_id_rtm(2), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     !--- surface temperature
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(3),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "surface_temperature_modeled", &
                              "surface_temperature_modeled", &
                              "surface_temperature_modeled", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_tsfc, max_tsfc, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- bt3b_clear_rtm
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(4),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "bt3b_clear_rtm", &
                              "bt3b_clear_rtm", &
                              "bt3b_clear_rtm", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_t3b, max_t3b, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- bt3b_clear_solar_rtm
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(5),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "bt3b_clear_solar_rtm", &
                              "bt3b_clear_solar_rtm", &
                              "bt3b_clear_solar_rtm", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_t3b, max_t3b, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- bt4_clear_rtm
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(6),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "bt4_clear_rtm", &
                              "bt4_clear_rtm", &
                              "bt4_clear_rtm", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_t4, max_t4, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- bt5_clear_rtm
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(7),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "bt5_clear_rtm", &
                              "bt5_clear_rtm", &
                              "bt5_clear_rtm", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_t5, max_t5, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- etrop
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(8),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "emissivity_11um_tropopause", &
                              "emissivity_11um_tropopause", &
                              "emissivity_11um_tropopause", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_etropo, max_etropo, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- alb1_clear_rtm
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(9),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "alb1_clear_rtm", &
                              "alb1_clear_rtm", &
                              "alb1_clear_rtm", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_alb1, max_alb1, "%", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- alb2_clear_rtm
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(10),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "alb2_clear_rtm", &
                              "alb2_clear_rtm", &
                              "alb2_clear_rtm", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_alb2, max_alb2, "%", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- bt4_std_3x3
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(11),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "bt4_std_3x3", &
                              "bt4_std_3x3", &
                              "bt4_std_3x3", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_t4_std, max_t4_std, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- bt4_max_3x3
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(12),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "bt4_max_3x3", &
                              "bt4_max_3x3", &
                              "bt4_max_3x3", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_t4, max_t4, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- alb1_std_3x3
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(13),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "alb1_std_3x3", &
                              "alb1_std_3x3", &
                              "alb1_std_3x3", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_alb1_std, max_alb1_std, "%", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- alb1_min_3x3
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(14),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "alb1_min_3x3", &
                              "alb1_min_3x3", &
                              "alb1_min_3x3", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_alb1, max_alb1, "%", missing_value_real4, istatus)

     !--- bt45_bt4_max_3x3
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(15),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "bt45_bt4_max_3x3", &
                              "bt45_bt4_max_3x3", &
                              "bt45_bt4_max_3x3", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_t45, max_t45, "K", missing_value_real4, istatus)

     !--- ems3b
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(16),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "ems3b", &
                              "ems3b", &
                              "3.75um_toa_emissivity", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_ems3b, max_ems3b, "none", missing_value_real4, istatus)
     !--- ems3b clear
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(17),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "ems3b_clear", &
                              "ems3b_clear", &
                              "3.75um_toa_emissivity_estimate_clear_sky", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_ems3b, max_ems3b, "none", missing_value_real4, istatus)

     !--- ems3b median 3x3
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(18),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "ems3b_median_3x3", &
                              "ems3b_median_3x3", &
                              "3.75um_toa_emissivity_median_over_a_3x3_array", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_ems3b, max_ems3b, "none", missing_value_real4, istatus)

     !--- Zc_opaque_cloud
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(19),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "Zc_opaque_cloud", &
                              "Zc_opaque_cloud", &
                              "Cloud_height_assuming_an_opaque_cloud", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_zc, max_zc, "km", missing_value_real4, istatus)

     !--- bt3b
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(20),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "ch3b_temperature_median_3x3", &
                              "ch3b_temperature_median_3x3", &
                              "ch3b_temperature_median_filtered_over_a_3x3_array", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_t3b, max_t3b, "K", missing_value_real4, istatus)

     !--- ch3b surface emissivity
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(21),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "ch3b_surface_emissivity", &
                              "ch3b_surface_emissivity", &
                              "ch3b_surface_emissivity", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_sfc_ems, max_sfc_ems, "none", missing_value_real4, istatus)

     !--- oisst_uni
     call DEFINE_PIXEL_2D_SDS(sds_id_rtm(22),sd_id_rtm,sds_dims_2d,sds_chunk_size, &
                              "sst_background_uni_3x3", &
                              "sst_background_uni_3x3", &
                              "sst_background_uni_3x3", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_sst_std, max_sst_std, "K", missing_value_real4, istatus)

     !--- check for and report errors
     if (istatus_sum /= 0) then
       print *, EXE_PROMPT, MOD_PROMPT, "Error defining sds in rtm hdf file"
     endif


  endif


!---------------------------------------------------------
!-- define ash file structure
!---------------------------------------------------------
  if (ash_file_flag > 0) then
     file_ash = trim(file_1b)//".ash.hdf"
     print *, EXE_PROMPT, MOD_PROMPT, "creating ASH file ", trim(file_ash)
     sd_id_ash = sfstart(trim(dir_ash)//trim(file_ash),DFACC_CREATE)
     if (sd_id_ash < 0) then
       erstat = 68
       print *, EXE_PROMPT, MOD_PROMPT, "ASH file creation failed. Exiting..."
       stop 68
     endif

!--- write attributes
 call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_ash,"PIXEL",file_ash,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           blank_int4,blank_int4,blank_char,blank_real4, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, modis_clr_alb_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a1_3b, a2_3b, nu_3b, &
                           a1_4, a2_4, nu_4, a1_5, a2_5, nu_5, &
                           solar_3b_nu,timerr_seconds,sc_id_new)


!-- reset status flag for error checking
     istatus_sum = 0

!-- scan line variables
     sds_id_ash(1) = sfcreate(sd_id_ash,"scan_line_number",DFNT_INT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_ash(1), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_ash(1), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_ash(1), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     sds_id_ash(2) = sfcreate(sd_id_ash,"scan_line_time",DFNT_FLOAT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_ash(2), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_ash(2), "UNITS", DFNT_CHAR8, 5, "hours") + istatus_sum
     istatus_sum = sfsnatt(sds_id_ash(2), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum
     
     call DEFINE_PIXEL_2D_SDS(sds_id_ash(3),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                             "ash_mask", &
                             "ash_mask", &
                             "ash_mask", &
                             DFNT_INT16, sym%NO_SCALING, &
                              0.0, 0.0, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus
     
     call DEFINE_PIXEL_2D_SDS(sds_id_ash(4),sd_id_ash,sds_dims_2d,sds_chunk_size,  &
                              "ash_mass_loading", &
                              "ash_mass_loading", &
                              "ash_mass_loading", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              0.0, 200.0, "ton/km^2", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus
     
     call DEFINE_PIXEL_2D_SDS(sds_id_ash(5),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                              "ash_height", &
                              "ash_height", &
                              "ash_height", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_zc, max_zc, "km", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus
     
     call DEFINE_PIXEL_2D_SDS(sds_id_ash(6),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                             "ash_effective_radius", &
                             "ash_effective_radius", &
                             "ash_effective_radius", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              0.0, 15.0, "micron", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus
     
     call DEFINE_PIXEL_2D_SDS(sds_id_ash(7),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                             "ash_quality_flags", &
                             "ash_quality_flags", &
                             "ash_quality_flags", &
                             DFNT_INT8, sym%NO_SCALING, &
                              0.0, 0.0, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus
     
     IF (ash_file_flag == 2) THEN
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(8),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                                "ash_emissivity", &
                                "ash_emissivity", &
                                "ash_emissivity", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_ec, max_ec, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(9),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                                "ash_emissivity_tot", &
                                "ash_emissivity_tot", &
                                "ash_emissivity_tot", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_ec, max_ec, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(10),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                                "ash_beta1112", &
                                "ash_beta1112", &
                                "ash_beta1112", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_beta, max_beta, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(11),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                               "ash_beta1112_tot", &
                               "ash_beta1112_tot", &
                               "ash_beta1112_tot", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_beta, max_beta, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(12),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                               "ash_beta1112_tot_multi_low", &
                               "ash_beta1112_tot_multi_low", &
                               "ash_beta1112_tot_multi_low", &
                               DFNT_INT16, sym%LINEAR_SCALING, &
                                min_beta, max_beta, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(13),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                                "ash_beta1112_tot_multi_mid", &
                                "ash_beta1112_tot_multi_mid", &
                                "ash_beta1112_tot_multi_mid", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_beta, max_beta, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(14),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                               "ash_generic1", &
                               "ash_generic1", &
                               "ash_generic1", &
                               DFNT_FLOAT32, sym%NO_SCALING, &
                               0.0, 0.0, "unknown", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(15),sd_id_ash,sds_dims_2d,sds_chunk_size,& 
                                "ash_generic2", &
                                "ash_generic2", &
                                "ash_generic2", &
                                DFNT_FLOAT32, sym%NO_SCALING, &
                                0.0, 0.0, "unknown", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus

       call DEFINE_PIXEL_2D_SDS(sds_id_ash(16),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                                "ash_temperature", &
                                "ash_temperature", &
                                "ash_temperature", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_tc, max_tc, "K", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus

       call DEFINE_PIXEL_2D_SDS(sds_id_ash(17),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                                "ash_vis_optical_depth", &
                                "ash_vis_optical_depth", &
                                "ash_vis_optical_depth", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                0.0, 100.0, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus

       call DEFINE_PIXEL_2D_SDS(sds_id_ash(18),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                                "ash_pressure", &
                                "ash_pressure", &
                                "ash_pressure", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_pc, max_pc, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(19),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                                "ash_beta1112_opaque", &
                                "ash_beta1112_opaque", &
                                "ash_beta1112_opaque", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_beta, max_beta, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(20),sd_id_ash,sds_dims_2d,sds_chunk_size,& 
                                "ash_emissivity_error", &
                                "ash_emissivity_error", &
                                "ash_emissivity_error", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_ec_std, max_ec_std, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(21),sd_id_ash,sds_dims_2d,sds_chunk_size,& 
                                "ash_temperature_error", &
                                "ash_temperature_error", &
                                "ash_temperature_error", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_tc_std, max_tc_std, "K", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
     
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(22),sd_id_ash,sds_dims_2d,sds_chunk_size,& 
                                "ash_beta1112_error", &
                                "ash_beta1112_error", &
                                "ash_beta1112_error", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                min_beta_std, max_beta_std, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
       
       call DEFINE_PIXEL_2D_SDS(sds_id_ash(23),sd_id_ash,sds_dims_2d,sds_chunk_size, &
                                "ash_probability", &
                                "ash_probability", &
                                "ash_probability", &
                                DFNT_INT16, sym%LINEAR_SCALING, &
                                0.0, 1.0, "none", missing_value_real4, istatus)
       istatus_sum = istatus_sum + istatus
       
     ENDIF

!--- check for and report errors
     if (istatus_sum /= 0) then
       print *, EXE_PROMPT, MOD_PROMPT, "Error defining sds in ash hdf file"
     endif

  endif

!---------------------------------------------------------
!-- define level2 file structure
!---------------------------------------------------------
  if (level2_file_flag == sym%YES) then
     file_level2= trim(file_1b)//".level2.hdf"
     print *, EXE_PROMPT, MOD_PROMPT, "creating level-2 file ", trim(file_level2)
     sd_id_level2 = sfstart(trim(dir_level2)//trim(file_level2),DFACC_CREATE)
     if (sd_id_level2 < 0) then
       erstat = 68
       print *, EXE_PROMPT, MOD_PROMPT, "Level-2 file creation failed. Exiting..."
       stop 68
     endif

!--- write attributes
 call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_level2,"PIXEL",file_level2,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           blank_int4,blank_int4,blank_char,blank_real4, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, modis_clr_alb_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a1_3b, a2_3b, nu_3b, &
                           a1_4, a2_4, nu_4, a1_5, a2_5, nu_5, &
                           solar_3b_nu,timerr_seconds,sc_id_new)


     !-- reset status flag for error checking
     istatus_sum = 0

     !-- scan line variables

     !--- scan line number
     sds_id_level2(sds_num_level2_scanline) = sfcreate(sd_id_level2,"scan_line_number",DFNT_INT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_level2(sds_num_level2_scanline), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_level2(sds_num_level2_scanline), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     standard_name_temp = "scan_line_number"
     istatus_sum = sfscatt(sds_id_level2(sds_num_level2_scanline), "STANDARD_NAME", DFNT_CHAR8,  &
                   len_trim(standard_name_temp), trim(standard_name_temp)) + istatus_sum
     long_name_temp = "scan line number"
     istatus_sum = sfscatt(sds_id_level2(sds_num_level2_scanline), "LONG_NAME", DFNT_CHAR8, len_trim(long_name_temp), trim(long_name_temp)) + istatus_sum
     istatus_sum = sfsnatt(sds_id_level2(sds_num_level2_scanline), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     !--- scanline time
     sds_id_level2(sds_num_level2_time) = sfcreate(sd_id_level2,"scan_line_time",DFNT_FLOAT32,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_level2(sds_num_level2_time), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_level2(sds_num_level2_time), "UNITS", DFNT_CHAR8, 5, "hours_utc") + istatus_sum
     long_name_temp = "time for the scan line in fractional hours"
     istatus_sum = sfscatt(sds_id_level2(sds_num_level2_time), "LONG_NAME", DFNT_CHAR8, len_trim(long_name_temp), trim(long_name_temp)) + istatus_sum
     istatus_sum = sfsnatt(sds_id_level2(sds_num_level2_time), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     !--- bad scan flag
     sds_id_level2(sds_num_level2_bad_scan) = sfcreate(sd_id_level2,"bad_scan_line_flag",DFNT_INT8,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_level2(sds_num_level2_bad_scan), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_level2(sds_num_level2_bad_scan), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_level2(sds_num_level2_bad_scan), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     !--- asc des flag
     sds_id_level2(sds_num_level2_asc_flag) = sfcreate(sd_id_level2,"asc_des_flag",DFNT_INT8,1,sds_dims_1d)
     istatus_sum = sfsnatt(sds_id_level2(sds_num_level2_asc_flag), "SCALED", DFNT_INT8, 1, sym%NO_SCALING) + istatus_sum
     istatus_sum = sfscatt(sds_id_level2(sds_num_level2_asc_flag), "UNITS", DFNT_CHAR8, 4, "none") + istatus_sum
     istatus_sum = sfsnatt(sds_id_level2(sds_num_level2_asc_flag), "RANGE_MISSING", DFNT_FLOAT32, 1, missing_value_real4) + istatus_sum

     !---  packed pixel quality meta-data
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_meta_data),sd_id_level2,sds_dims_2d,sds_chunk_size,&
                              "packed_pixel_meta_data", &
                              "packed_pixel_meta_data", &
                              "pixel quality flags packed into one byte", &
                              DFNT_INT8, sym%NO_SCALING, 0.0, 0.0, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- latitude
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_lat),sd_id_level2,anchor_sds_dims_2d,anchor_sds_chunk_size, &
                              "anchor_latitude", &
                              "latitude", &
                              "latitude vector reported at the 51 level-1b anchor values", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_lat, max_lat, "degrees_north", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- longitude
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_lon),sd_id_level2,anchor_sds_dims_2d,anchor_sds_chunk_size, &
                              "anchor_longitude", &
                              "longitude", &
                              "longitude vector reported at the 51 level-1b anchor values", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_lon, max_lon, "degrees_east", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- sensor zenith
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_zen),sd_id_level2,anchor_sds_dims_2d,anchor_sds_chunk_size, &
                              "anchor_sensor_zenith", &
                              "sensor_zenith_angle", &
                              "sensor zenith view angle in degrees reported at the 51 level-1b anchor values", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_ang, max_ang, "degrees", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- solar zenith
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_solzen),sd_id_level2,anchor_sds_dims_2d,anchor_sds_chunk_size, &
                              "anchor_solar_zenith", &
                              "solar_zenith_angle", &
                              "solar zenith angle in degrees reported at the 51 level-1b anchor values", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_ang, max_ang, "degrees", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- relative azimuth
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_relaz),sd_id_level2,anchor_sds_dims_2d,anchor_sds_chunk_size, &
                              "anchor_relative_azimuth", &
                              "relative_azimuth_angle", &
                              "relative azimuth angle in degrees reported at the 51 level-1b anchor values", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_ang, max_ang, "degrees", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- solar azimuth
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_solaz),sd_id_level2,anchor_sds_dims_2d,anchor_sds_chunk_size, &
                              "anchor_solar_azimuth", &
                              "solar_azimuth_angle", &
                              "solar azimuth angle in degrees reported at the 51 level-1b anchor values", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_solaz, max_solaz, "degrees", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !---  packed land cover
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_land),sd_id_level2,sds_dims_2d,sds_chunk_size,&
                              "packed_land_cover", &
                              "packed_land_cover", &
                              "land cover, snow and coast values packed into one byte", &
                              DFNT_INT8, sym%NO_SCALING, 0.0, 0.0, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus


!----- START OBSERVATIONS

     !--- ch1 reflectance
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ch1),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "ch1_reflectance", &
                              "toa_bidirectional_reflectance_0_63_micron", &
                              "observed reflectance at 0.63_micron as a percentage using PATMOS-x calibration", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_alb1, max_alb1, "%", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- ch2 reflectance
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ch2),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "ch2_reflectance", &
                              "toa_bidirectional_reflectance_0_86_micron", &
                              "observed_reflectance_0.86_micron expressed as a percentage using PATMOS-x calibration", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_alb2, max_alb2, "%", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- ch3a reflectance
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ch3a),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "ch3a_reflectance", &
                              "toa_bidirectional_reflectance_1_60_micron", &
                              "observed_reflectance_1.60_micron expressed as a percentage using PATMOS-x calibration", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_alb3a, max_alb3a, "%", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- ch3b reflectance
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ch3b),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "ch3b_reflectance", &
                              "toa_bidirectional_pseudo_reflectance_3_75_micron", &
                              "observed_pseudo_reflectance_3.75_micron expressed as a percentage using PATMOS-x calibration", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_alb3b, max_alb3b, "%", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- ch3b - ch4 temperature
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ch3b4),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "ch3b-ch4_temperature_difference", &
                              "toa_brightness_temperature_difference_3_75_minus_11_micron", &
                              "toa brightness temperature difference between 3.75 and 11 micron using PATMOS-x calibration", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_btd_3b_4, max_btd_3b_4, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- ch4 temperature
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ch4),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "ch4_temperature", &
                              "toa_brightness_temperature_11_micron", &
                              "toa_brightness_temperature_11_micron using PATMOS-x calibration", &
                              DFNT_INT16, sym%LINEAR_SCALING, &
                              min_t4, max_t4, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- ch4 - ch5 temperature
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ch45),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "ch4-ch5_temperature_difference", &
                              "toa_brightness_temperature_difference_11_minus_12_micron", &
                              "toa_brightness_temperature_difference_between_11_and_12_microns using PATMOS-x calibration", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_t45, max_t45, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- ch1_ref_std_3x3
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ch1_std),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "ch1_reflectance_stddev_3x3", &
                              "toa_reflectance_0_63_micron_3x3_standard_deviation", &
                              "standard_deviation of the 0.63 micron reflectance computed over a 3x3 pixel array using PATMOS-x calibration", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_alb1_std, max_alb1_std, "%", missing_value_real4, istatus)
     !--- bt4_std_3x3
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ch4_std),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "ch4_temperature_stddev_3x3", &
                              "toa_brightness_temperature_11_micron_3x3_standard_deviation", &
                              "standard_deviation of the 11 micron BT computed over a 3x3 pixel array using PATMOS-x calibration", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_t4_std, max_t4_std, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!---- Cloud Properties

     !--- cloud probability
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_cldprob),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "cloud_probability", &
                              "cloud_probability", &
                              "probability of a pixel being cloudy from the Bayesian cloud mask", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_frac, max_frac, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

if (ncdc_level2_flag == sym%NO) then

     !--- cloud type
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ctype),sd_id_level2,sds_dims_2d,sds_chunk_size,&
                              "cloud_type", &
                              "cloud_type", &
                              "integer classification of the cloud type including clear and aerosol type", &
                              DFNT_INT8, sym%NO_SCALING, 0.0, 0.0, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- cloud pressure from the split-window approach
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ctp),sd_id_level2,sds_dims_2d,sds_chunk_size,&
                              "cld_press_sw", &
                              "air_pressure_at_cloud_top", &
                              "cloud-top pressure computed using the split-window approach", &
                              DFNT_INT8, sym%LINEAR_SCALING, min_pc, max_pc, &
                              "hPa", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- cloud temperature from the split-window approach
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ctt),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "cld_temp_sw", &
                              "air_temperature_at_cloud_top", &
                              "cloud-top temperature computed using the split-window approach", &
                              DFNT_INT8, sym%LINEAR_SCALING, min_tc, max_tc, &
                              "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- cloud emissivity from the split window approach
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ec),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                             "cld_emiss_sw", &
                             "cloud_emissivity", &
                             "cloud emissivity at 11 micron determined from the split-window approach", &
                             DFNT_INT8, sym%LINEAR_SCALING, min_ec, max_ec, &
                             "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- beta from the split-window approach
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_beta),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "cld_beta_sw", &
                              "cloud_beta_11_12", &
                              "cloud 11/12 micron beta value determined from the split-window method", &
                              DFNT_INT8, sym%LINEAR_SCALING, min_beta, max_beta, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- cloud optical depth from the vis/nir approach
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_cod),sd_id_level2,sds_dims_2d, sds_chunk_size,&
                              "cld_opd_vis", &
                              "atmosphere_optical_thickness_due_to_cloud", &
                              "cloud_optical_thickness_at_0.63_micron", &
                              DFNT_INT16, sym%LINEAR_SCALING, min_tau, max_tau, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- cloud effective particle radius from the vis/nir approach
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ceps),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "cld_reff_vis", &
                              "effective_radius_of_cloud_particle", &
                              "effective_radius_of_cloud_particle_determined_from_0.63_and_3.75_micron_observations", &
                              DFNT_INT16, sym%LINEAR_SCALING, min_re, max_re, &
                              "micron", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- cloud albedo
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_cldalb),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "cloud_albedo", &
                              "cloud_albedo", &
                              "cloud_albedo", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_albedo, max_albedo, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- cloud transmission
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_cldtrn),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "cloud_transmission", &
                              "cloud_transmission", &
                              "cloud_transmission", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_transmission, max_transmission, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus
 
     !--- cloud fraction
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_cldfrac),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "cloud_fraction", &
                              "cloud_fraction", &
                              "cloud_fraction_computed_over_a_3x3_array", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_frac, max_frac, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- cloud fraction uncertainity
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_cldfracuncer),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "cloud_fraction_uncertainty", &
                              "cloud_fraction_uncertainty", &
                              "cloud_fraction_uncertainty_computed_over_a_3x3_array", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_frac, max_frac, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus


!---- Non-cloud Properties

     !--- etrop
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_etrop),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "emissivity_11um_tropopause", &
                              "cloud_emissivity_at_tropopause", &
                              "emissivity_at_11_microns_assuming_the_cloud_was_located_at_the_Tropopause", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_etropo, max_etropo, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- ch 1 aerosol optical thickness
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_aot1),sd_id_level2,sds_dims_2d, sds_chunk_size, &
                              "aot_ch1", &
                              "optical_thickness_of_atmosphere_layer_due_to_aerosol_0.63_micron", &
                              "optical_thickness_of_atmosphere_layer_due_to_aerosol_0.63_micron", &
                              DFNT_INT16, sym%LINEAR_SCALING,min_aot, max_aot, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- outgoing longwave radiation
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_olr),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "olr", &
                              "outgoing_longwave_radiation", &
                              "outgoing_longwave_radiation", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_olr, max_olr, "W/m^2", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- ndvi_sfc
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_ndvi),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "ndvi_sfc", &
                              "normalized_difference_vegetation_index_at_surface", &
                              "normalized difference vegetation index atmospherically corrected", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_ndvi, max_ndvi, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- surface temperature
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_tsfc),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "surface_temperature_retrieved", &
                              "surface_temperature_retrieved", &
                              "surface_temperature_retrieved", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_tsfc, max_tsfc, "K", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- remote sensing reflectance
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_rsr),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "remote_sensing_reflectance", &
                              "remote_sensing_reflectance", &
                              "remote_sensing_reflectance", &
                              DFNT_INT8, sym%LINEAR_SCALING, &
                              min_rsr, max_rsr, "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

!---- quality flags
     !--- first level2 quality flags
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_qf1),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "quality_flags_1", &
                              "quality_flags_1", &
                              "first set of packed quality flags", &
                              DFNT_INT8, sym%NO_SCALING, 0.0, 0.0, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

     !--- second level2 quality flags
     call DEFINE_PIXEL_2D_SDS(sds_id_level2(sds_num_level2_qf2),sd_id_level2,sds_dims_2d,sds_chunk_size, &
                              "quality_flags_2", &
                              "quality_flags_2", &
                              "second set of packed quality flags", &
                              DFNT_INT8, sym%NO_SCALING, 0.0, 0.0, &
                              "none", missing_value_real4, istatus)
     istatus_sum = istatus_sum + istatus

endif  !end of ncdc_level2_flag check
 
     !--- check for and report errors
     if (istatus_sum /= 0) then
       print *, EXE_PROMPT, MOD_PROMPT, "Error defining sds in level2 hdf file"
       stop
     endif

  endif

end subroutine DEFINE_HDF_FILE_STRUCTURES

!====================================================================
! SUBROUTINE Name: WRITE_PIXEL_HDF_RECORDS
!
! Function:
!   Writes output to appropriate Level 2 files
!
! Description:
!   This subroutine, given the flags that determine which files are 
!   being created, outputs the various pixel level outputs to the
!   appropriate level 2 files and appropriate SDSs for a given segment
!
!====================================================================
subroutine WRITE_PIXEL_HDF_RECORDS(cmr_file_flag, &
                                   nav_file_flag, &
                                   sst_file_flag, &
                                   cld_file_flag, &
                                   obs_file_flag, &
                                   geo_file_flag, &
                                   rtm_file_flag, &
                                   ash_file_flag, &
                                   level2_file_flag, &
                                   iseg)

 integer, intent(in):: cmr_file_flag
 integer, intent(in):: nav_file_flag
 integer, intent(in):: obs_file_flag
 integer, intent(in):: geo_file_flag
 integer, intent(in):: rtm_file_flag
 integer, intent(in):: ash_file_flag
 integer, intent(in):: level2_file_flag
 integer, intent(in):: sst_file_flag
 integer, intent(in):: cld_file_flag
 integer, intent(in):: iseg
 integer:: i
 integer:: j
 integer:: istatus

! HDF function declarations
 integer:: sfwdata
 integer:: erstat

!-----------------------------------------------------------------------
! Get time of each scan line and convert to scale
!-----------------------------------------------------------------------
  utc_scan_time_hours = scan_time/60.0/60.0/1000.0

!--------------------------------------------------------------------------------
! determine start and edges for writing this segment's data to pixel hdf file
!-----------------------------------------------------------------------------
    sds_start(1) = 0     !pixel dimension
    sds_start(2) = num_scans_hdf

    sds_stride(1) = 1
    sds_stride(2) = 1

    sds_edge(1) = num_pix
    sds_edge(2) = min(num_scans_read,num_scans - sds_start(2))

    if (sds_edge(2) <= 0) then
      return
    endif

    anchor_sds_edge(1) = num_anchors
    anchor_sds_edge(2) = min(num_scans_read,num_scans - sds_start(2))

!--- update num_scans_hdf
    num_scans_hdf = min(num_scans,num_scans_hdf + num_scans_read)

!-----------------------------------------------------------------------
! write to cmr (cld mask results) file
!-----------------------------------------------------------------------
    if (cmr_file_flag == sym%YES) then
!--- store fire results in blank cld type value (7)
       temp_mask = cld_type
       where((cld_mask < 2) .and. (fire == 1))
            temp_mask = 7
      endwhere
      one_byte_temp = cld_test_vector(1,:,:) + ishft(cld_test_vector(8,:,:),1) + ishft(dust,2) + &  
                         ishft(temp_mask,3) + ishft(cld_mask,6)
     
     istatus = 0

     istatus = sfwdata(sds_id_cmr(1), sds_start(2), sds_stride(2),          &
                       sds_edge(2), scan_number(j_min:sds_edge(2) + j_min - 1)) + istatus

     istatus = sfwdata(sds_id_cmr(2), sds_start(2), sds_stride(2), sds_edge(2),  &
                       utc_scan_time_hours(j_min:sds_edge(2) + j_min - 1)) + istatus

     istatus = sfwdata(sds_id_cmr(3), sds_start, sds_stride, sds_edge, &
                       one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      istatus = sfwdata(sds_id_cmr(4), sds_start, sds_stride, sds_edge, &
                        cld_test_vector_packed(1, :, j_min:sds_edge(2) + j_min - 1)) + istatus

      istatus = sfwdata(sds_id_cmr(5), sds_start, sds_stride, sds_edge, &
                        cld_test_vector_packed(2, :, j_min:sds_edge(2) + j_min - 1)) + istatus

      istatus = sfwdata(sds_id_cmr(6), sds_start, sds_stride, sds_edge, &
                        cld_test_vector_packed(3, :, j_min:sds_edge(2) + j_min - 1)) + istatus

      istatus = sfwdata(sds_id_cmr(7), sds_start, sds_stride, sds_edge, &
                        cld_test_vector_packed(4, :, j_min:sds_edge(2) + j_min - 1)) + istatus


!--- check status
     if (istatus /= 0) then
       print *, EXE_PROMPT, MOD_PROMPT, "error on write to cmr file ", istatus
       erstat = 75
       stop 75
     endif
    endif

!-------------------------------------------------------------------------
! write to sst file
!-------------------------------------------------------------------------
   if (sst_file_flag == sym%YES) then

!--- reset status
      istatus = 0

!--- scan number
      istatus = sfwdata(sds_id_sst(1), sds_start(2), sds_stride(2),          &
                sds_edge(2), scan_number(j_min:sds_edge(2) + j_min - 1)) + istatus

!--- scan time 
      istatus = sfwdata(sds_id_sst(2), sds_start(2), sds_stride(2), sds_edge(2),  &
                        utc_scan_time_hours(j_min:sds_edge(2) + j_min - 1)) + istatus

!--  observed sst
     call SCALE_VECTOR_I2_RANK2(tsfc,sym%LINEAR_SCALING,min_tsfc,max_tsfc,missing_value_real4,two_byte_temp)

     istatus = sfwdata(sds_id_sst(3), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- packed cld mask (cld mask,snow,glint,coast,land,day,valid)
     one_byte_temp = cld_test_vector(1,:,:) + ishft(cld_test_vector(2,:,:),1) + ishft(land_mask,2) + &
         ishft(coast_mask,3) + ishft(glint_mask,4) + &
         ishft(cld_test_vector(8,:,:),5) + ishft(cld_mask,6)

     istatus = sfwdata(sds_id_sst(4), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!-- sst analysis
     call SCALE_VECTOR_I1_RANK2(sst_anal,sym%LINEAR_SCALING,min_sst,max_sst,missing_value_real4,one_byte_temp)
     istatus = sfwdata(sds_id_sst(5), sds_start, sds_stride, sds_edge, &
                      one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- sst quality flag
     where (tsfc == missing_value_real4)
       sst_qf = missing_value_int1
     end where
     istatus = sfwdata(sds_id_sst(6), sds_start, sds_stride, sds_edge, &
                      sst_qf(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!-- sst already masked
    call SCALE_VECTOR_I2_RANK2(sst_masked,sym%LINEAR_SCALING,min_sst,max_sst,missing_value_real4,two_byte_temp)
    istatus = sfwdata(sds_id_sst(7), sds_start, sds_stride, sds_edge, &
                      two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!---- check for errors
      if (istatus /= 0) then
         print *, EXE_PROMPT, MOD_PROMPT, "Error on write to sst file ", istatus
         erstat = 77
         stop 77
      endif
   endif

!-------------------------------------------------------------------------
! write to nav file  ***CHANGE PACKING OF LAT/LON ****
!-------------------------------------------------------------------------
   if (nav_file_flag == sym%YES) then

      istatus = 0

!--- scan number
      istatus = sfwdata(sds_id_nav(1), sds_start(2), sds_stride(2),          &
                sds_edge(2), scan_number(j_min:sds_edge(2) + j_min - 1)) + istatus

!--- time
      istatus = sfwdata(sds_id_nav(2), sds_start(2), sds_stride(2), sds_edge(2),  &
                utc_scan_time_hours(j_min:sds_edge(2) + j_min - 1)) + istatus

!--- ascending/descending flag
      istatus = sfwdata(sds_id_nav(3), sds_start(2), sds_stride(2), sds_edge(2),  &
                ascend(j_min:sds_edge(2) + j_min - 1)) + istatus

!--- bad scan flag
      istatus = sfwdata(sds_id_nav(4), sds_start(2), sds_stride(2), sds_edge(2),  &
                bad_scan_flag(j_min:sds_edge(2) + j_min - 1)) + istatus

!--- latitude
      call SCALE_VECTOR_I2_RANK2(lat,sym%LINEAR_SCALING,min_lat,max_lat,missing_value_real4,two_byte_temp)
      istatus = sfwdata(sds_id_nav(5),sds_start,sds_stride,sds_edge, &
                   two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- longitude
      call SCALE_VECTOR_I2_RANK2(lon,sym%LINEAR_SCALING,min_lon,max_lon,missing_value_real4,two_byte_temp)
      istatus = sfwdata(sds_id_nav(6),sds_start,sds_stride,sds_edge,  &
                   two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!---- report status
      if (istatus /= 0) then
         print *, EXE_PROMPT, MOD_PROMPT, "error on write to nav file ", istatus
         erstat = 78
         stop 78
      endif
   endif

!-------------------------------------------------------------------------
! write to cld product file
!-------------------------------------------------------------------------
   if (cld_file_flag == sym%YES) then

!--- reset error counter
      istatus = 0

!--- scan number
      istatus = sfwdata(sds_id_cld(1), sds_start(2), sds_stride(2), sds_edge(2),  &
                        scan_number(j_min:sds_edge(2) + j_min - 1)) + istatus

!--- scan time
      istatus = sfwdata(sds_id_cld(2), sds_start(2), sds_stride(2), sds_edge(2),  &
                        utc_scan_time_hours(j_min:sds_edge(2) + j_min - 1)) + istatus

!--- cld mask
   istatus = sfwdata(sds_id_cld(3), sds_start,sds_stride,sds_edge,       &
                        cld_mask(:,j_min:sds_edge(2) + j_min - 1)) + istatus

!--- cld type
   istatus = sfwdata(sds_id_cld(4), sds_start,sds_stride,sds_edge,     &
                        cld_type(:,j_min:sds_edge(2) + j_min - 1)) + istatus

!--- cld top temperature from split window
   call SCALE_VECTOR_I1_RANK2(Tc_sw,sym%LINEAR_SCALING,min_tc,max_tc,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_cld(5), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- cld emissivity from split window
   call SCALE_VECTOR_I1_RANK2(ec_sw,sym%LINEAR_SCALING,min_ec,max_ec,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_cld(6), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- beta from split window
   call SCALE_VECTOR_I1_RANK2(beta_sw,sym%LINEAR_SCALING,min_beta,max_beta,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_cld(7), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- cld height
   call SCALE_VECTOR_I1_RANK2(Zc_sw,sym%LINEAR_SCALING,min_zc,max_zc,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_cld(8), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- sw packed cld product quality flags
   one_byte_temp = 0
   one_byte_temp = ishft(tc_sw_qf,4) + ishft(ec_sw_qf,2) + beta_sw_qf
   istatus = sfwdata(sds_id_cld(9), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- cld optical depth from vis
!  call SCALE_VECTOR_I1_RANK2(tau_vis,sym%SQUARE_ROOT_SCALING,min_tau,max_tau,missing_value_real4,one_byte_temp)
   call SCALE_VECTOR_I2_RANK2(tau_vis,sym%SQUARE_ROOT_SCALING,min_tau,max_tau,missing_value_real4,two_byte_temp)
   istatus = sfwdata(sds_id_cld(10), sds_start, sds_stride, sds_edge, &
                     two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- cld particle size from vis
!  call SCALE_VECTOR_I1_RANK2(re_vis,sym%SQUARE_ROOT_SCALING,min_re,max_re,missing_value_real4,one_byte_temp)
   call SCALE_VECTOR_I2_RANK2(re_vis,sym%SQUARE_ROOT_SCALING,min_re,max_re,missing_value_real4,two_byte_temp)
   istatus = sfwdata(sds_id_cld(11), sds_start, sds_stride, sds_edge, &
!                    one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
                     two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- vis packed cld product quality flags
   one_byte_temp = 0
   one_byte_temp = ishft(tau_vis_qf,2) + re_vis_qf
   istatus = sfwdata(sds_id_cld(12), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- infrared cld type
      istatus = sfwdata(sds_id_cld(13), sds_start,sds_stride,sds_edge,     &
                        cld_type_ir(:,j_min:sds_edge(2) + j_min - 1)) + istatus

!--- cld pressure
   call SCALE_VECTOR_I1_RANK2(Pc_sw,sym%LINEAR_SCALING,min_pc,max_pc,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_cld(14), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- cld top temperature from split window
   call SCALE_VECTOR_I1_RANK2(Tc_sw_error,sym%LINEAR_SCALING,min_tc_error,max_tc_error,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_cld(15), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- report status
    if (istatus /= 0) then
         print *, EXE_PROMPT, MOD_PROMPT, "error on write to cld file ", istatus
         erstat = 79
         stop 79
      endif
   endif

!-------------------------------------------------------------------------
! write to obs file
!-------------------------------------------------------------------------
   if (obs_file_flag == sym%YES) then

!--- scan line information
     istatus = 0

     istatus = sfwdata(sds_id_obs(1), sds_start(2), sds_stride(2),          &
                       sds_edge(2), scan_number(j_min:sds_edge(2) + j_min - 1)) + istatus

     istatus = sfwdata(sds_id_obs(2), sds_start(2), sds_stride(2), sds_edge(2),  &
                       utc_scan_time_hours(j_min:sds_edge(2) + j_min - 1)) + istatus

     istatus = sfwdata(sds_id_obs(3), sds_start(2), sds_stride(2), sds_edge(2),  &
                       bad_scan_flag(j_min:sds_edge(2) + j_min - 1)) + istatus

!-- ch1 reflectance
     temp_pix_array = alb1
     if (output_scaled_reflectances == sym%YES) then
       do j = j_min, j_min + num_scans_read -1
          temp_pix_array(:,j) = alb1(:,j) * cossolzen(:,j)
          if (bad_scan_flag(j) == sym%YES) then
            temp_pix_array(:,j) = missing_value_real4
          endif
          where (alb1(:,j) == missing_value_real4) 
            temp_pix_array(:,j) = missing_value_real4
          endwhere
       enddo
     endif

     call SCALE_VECTOR_I2_RANK2(temp_pix_array,sym%SQUARE_ROOT_SCALING,min_alb1,max_alb1,missing_value_real4,two_byte_temp)
     istatus = sfwdata(sds_id_obs(4), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!-- ch2 reflectance

     temp_pix_array = alb2
     if (output_scaled_reflectances == sym%YES) then
       do j = j_min, j_min + num_scans_read -1
          temp_pix_array(:,j) = alb2(:,j) * cossolzen(:,j)
          if (bad_scan_flag(j) == sym%YES) then
            temp_pix_array(:,j) = missing_value_real4
          endif
          where (alb2(:,j) == missing_value_real4) 
            temp_pix_array(:,j) = missing_value_real4
          endwhere
       enddo
     endif

     call SCALE_VECTOR_I1_RANK2(temp_pix_array,sym%SQUARE_ROOT_SCALING,min_alb2,max_alb2,missing_value_real4,one_byte_temp)
     istatus = sfwdata(sds_id_obs(5), sds_start, sds_stride, sds_edge, &
                       one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus


!--- ch3ab reflectance

!--- combine alb3a and alb3b together
     temp_pix_array = missing_value_real4
     do j = j_min, j_min + num_scans_read -1
       if (ch3a_on(j) == 1) then
         temp_pix_array(:,j) = alb3a(:,j)
         if (output_scaled_reflectances == sym%YES) then
          temp_pix_array(:,j) = alb3a(:,j) * cossolzen(:,j)
          where (alb3a(:,j) == missing_value_real4) 
            temp_pix_array(:,j) = missing_value_real4
          endwhere
         endif
       else
          temp_pix_array(:,j) = alb3b(:,j)
       endif
       if (bad_scan_flag(j) == sym%YES) then
          temp_pix_array(:,j) = missing_value_real4
       endif
     enddo

     call SCALE_VECTOR_I1_RANK2(temp_pix_array,sym%SQUARE_ROOT_SCALING,min_alb3b,max_alb3a,missing_value_real4,one_byte_temp)
     istatus = sfwdata(sds_id_obs(6), sds_start, sds_stride, sds_edge, &
                       one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- ch3b temperature
     call SCALE_VECTOR_I2_RANK2(bt3b,sym%LINEAR_SCALING,min_t3b,max_t3b,missing_value_real4,two_byte_temp)
     istatus = sfwdata(sds_id_obs(7), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- ch4 temperature
     call SCALE_VECTOR_I2_RANK2(bt4,sym%LINEAR_SCALING,min_t4,max_t4,missing_value_real4,two_byte_temp)
     istatus = sfwdata(sds_id_obs(8), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- ch5 temperature
     call SCALE_VECTOR_I2_RANK2(bt5,sym%LINEAR_SCALING,min_t5,max_t5,missing_value_real4,two_byte_temp)
     istatus = sfwdata(sds_id_obs(9), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- check for and report errors
     if (istatus /= 0) then
        print *, EXE_PROMPT, MOD_PROMPT, "Error writing to obs file: ", istatus
     endif
     
   endif

!-------------------------------------------------------------------------
! write to geo file
!-------------------------------------------------------------------------
   if (geo_file_flag == sym%YES) then

!--- scan line information
      istatus = 0

      istatus = sfwdata(sds_id_geo(1), sds_start(2), sds_stride(2),          &
                        sds_edge(2), scan_number(j_min:sds_edge(2) + j_min - 1)) + istatus

      istatus = sfwdata(sds_id_geo(2), sds_start(2), sds_stride(2), sds_edge(2),  &
                        utc_scan_time_hours(j_min:sds_edge(2) + j_min - 1)) + istatus

!---- pixel level data

!--- surface type (land cover classification)
   istatus = sfwdata(sds_id_geo(3), sds_start, sds_stride, sds_edge, sfc_type)+istatus

!--- surface elevation
   call SCALE_VECTOR_I2_RANK2(zsfc,sym%LINEAR_SCALING,min_z,max_z,missing_value_real4,two_byte_temp)
   istatus = sfwdata(sds_id_geo(4), sds_start, sds_stride, sds_edge, &
                     two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- sensor zenith ne
   call SCALE_VECTOR_I1_RANK2(satzen,sym%LINEAR_SCALING,min_ang,max_ang,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_geo(5), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
                                                                                                                                                                                   
!--- solar zenith
   call SCALE_VECTOR_I1_RANK2(solzen,sym%LINEAR_SCALING,min_ang,max_ang,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_geo(6), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- relative azimuth
   call SCALE_VECTOR_I1_RANK2(relaz,sym%LINEAR_SCALING,min_ang,max_ang,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_geo(7), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- coast (distance in km from a coastline)
   istatus = sfwdata(sds_id_geo(8), sds_start, sds_stride, sds_edge, coast)+istatus

!--- solar azimuth
   call SCALE_VECTOR_I1_RANK2(solaz,sym%LINEAR_SCALING,min_solaz,max_solaz,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_geo(9), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- land classification
   one_byte_temp = int(land,kind=int1)
   istatus = sfwdata(sds_id_geo(10), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- snow/ice mask
   one_byte_temp = int(snow,kind=int1)
   istatus = sfwdata(sds_id_geo(11), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- check for and report errors
     if (istatus /= 0) then
        print *, EXE_PROMPT, MOD_PROMPT, "Error writing to geo file: ", istatus
     endif

 endif

!-------------------------------------------------------------------------
! write to rtm file
!-------------------------------------------------------------------------
   if (rtm_file_flag == sym%YES) then

   !--- reset status flag
   istatus = 0

   !--- scan line number
   istatus = sfwdata(sds_id_rtm(1), sds_start(2), sds_stride(2),          &
                        sds_edge(2), scan_number(j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- scan line time
   istatus = sfwdata(sds_id_rtm(2), sds_start(2), sds_stride(2), sds_edge(2),  &
                        utc_scan_time_hours(j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- surface temperature
   call SCALE_VECTOR_I1_RANK2(tsfc_rtm,sym%LINEAR_SCALING,min_tsfc,max_tsfc,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(3), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- computed ch3b temperature
   call SCALE_VECTOR_I2_RANK2(bt_clear_ch3b_rtm,sym%LINEAR_SCALING,min_t3b,max_t3b,missing_value_real4,two_byte_temp)
   istatus = sfwdata(sds_id_rtm(4), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- computed ch3b temperature with solar
   call SCALE_VECTOR_I2_RANK2(bt_clear_ch3b_solar_rtm,sym%LINEAR_SCALING,min_t3b,max_t3b,missing_value_real4,two_byte_temp)
   istatus = sfwdata(sds_id_rtm(5), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- computed ch4 temperature
   call SCALE_VECTOR_I2_RANK2(bt_clear_ch4_rtm,sym%LINEAR_SCALING,min_t4,max_t4,missing_value_real4,two_byte_temp)
   istatus = sfwdata(sds_id_rtm(6), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- computed ch5 temperature
   call SCALE_VECTOR_I2_RANK2(bt_clear_ch5_rtm,sym%LINEAR_SCALING,min_t5,max_t5,missing_value_real4,two_byte_temp)
   istatus = sfwdata(sds_id_rtm(7), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- etrop
   call SCALE_VECTOR_I1_RANK2(emiss_11_tropo_rtm,sym%LINEAR_SCALING,min_etropo,max_etropo,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(8), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- alb1 clear
   call SCALE_VECTOR_I2_RANK2(alb1_clear,sym%LINEAR_SCALING,min_alb1,max_alb1,missing_value_real4,two_byte_temp)
   istatus = sfwdata(sds_id_rtm(9), sds_start, sds_stride, sds_edge, &
                     two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- alb2 clear
   call SCALE_VECTOR_I2_RANK2(alb2_clear,sym%LINEAR_SCALING,min_alb2,max_alb2,missing_value_real4,two_byte_temp)
   istatus = sfwdata(sds_id_rtm(10), sds_start, sds_stride, sds_edge, &
                     two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- bt4_std_3x3
   call SCALE_VECTOR_I1_RANK2(bt4_std_3x3,sym%LINEAR_SCALING,min_t4_std,max_t4_std,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(11), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- bt4_max_3x3
   call SCALE_VECTOR_I1_RANK2(bt4_max_3x3,sym%LINEAR_SCALING,min_t4,max_t4,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(12), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- alb1_std_3x3
   call SCALE_VECTOR_I1_RANK2(alb1_std_3x3,sym%LINEAR_SCALING,min_alb1_std,max_alb1_std,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(13), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- alb1_min_3x3
   call SCALE_VECTOR_I1_RANK2(alb1_min_3x3,sym%LINEAR_SCALING,min_alb1,max_alb1,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(14), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- bt45_bt4_max_3x3
   call SCALE_VECTOR_I1_RANK2(bt45_bt4_max_3x3,sym%LINEAR_SCALING,min_t45,max_t45,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(15), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- ems3b
   call SCALE_VECTOR_I1_RANK2(ems3b,sym%LINEAR_SCALING,min_ems3b,max_ems3b,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(16), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- ems3b clear-sky
   call SCALE_VECTOR_I1_RANK2(ems3b_clear_solar_rtm,sym%LINEAR_SCALING,min_ems3b,max_ems3b,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(17), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- ems3b median 3x3
   call SCALE_VECTOR_I1_RANK2(ems3b_median_3x3,sym%LINEAR_SCALING,min_ems3b,max_ems3b,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(18), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- Zc_opaque_cloud
   call SCALE_VECTOR_I1_RANK2(Zc_opaque_cloud,sym%LINEAR_SCALING,min_zc,max_zc,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(19), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- ch3b temperature
   call SCALE_VECTOR_I2_RANK2(bt3b_median_3x3,sym%LINEAR_SCALING,min_t3b,max_t3b,missing_value_real4,two_byte_temp)
   istatus = sfwdata(sds_id_rtm(20), sds_start, sds_stride, sds_edge, &
                     two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !--- ch3b surface emissiviy
   call SCALE_VECTOR_I1_RANK2(sfc_emiss_3b,sym%LINEAR_SCALING,min_sfc_ems,max_sfc_ems,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(21), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

   !-- background sst uniformity
   call SCALE_VECTOR_I1_RANK2(sst_anal_uni,sym%LINEAR_SCALING,min_sst_std,max_sst_std,missing_value_real4,one_byte_temp)
   istatus = sfwdata(sds_id_rtm(22), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus


   !--- check for and report errors
   if (istatus /= 0) then
      print *, EXE_PROMPT, MOD_PROMPT, "Error writing to rtm file: ", istatus
   endif

   endif

!-------------------------------------------------------------------------
! write to ash file
!-------------------------------------------------------------------------
   if (ash_file_flag > 0) then

      istatus = 0

      istatus = sfwdata(sds_id_ash(1), sds_start(2), sds_stride(2),          &
                        sds_edge(2), scan_number(j_min:sds_edge(2) + j_min - 1)) + istatus

      istatus = sfwdata(sds_id_ash(2), sds_start(2), sds_stride(2), sds_edge(2),  &
                        utc_scan_time_hours(j_min:sds_edge(2) + j_min - 1)) + istatus

     istatus = sfwdata(sds_id_ash(3), sds_start, sds_stride, sds_edge, &
                       ash_mask(:, j_min:sds_edge(2) + j_min - 1)) + istatus

     call SCALE_VECTOR_I2_RANK2(ash_mass_loading,sym%LINEAR_SCALING,0.0,200.0,missing_value_real4,two_byte_temp)
     istatus = sfwdata(sds_id_ash(4), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

     call SCALE_VECTOR_I2_RANK2(ash_height,sym%LINEAR_SCALING,min_zc,max_zc,missing_value_real4,two_byte_temp)
     istatus = sfwdata(sds_id_ash(5), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
     
     call SCALE_VECTOR_I2_RANK2(ash_effective_radius,sym%LINEAR_SCALING,0.0,15.0,missing_value_real4,two_byte_temp)
     istatus = sfwdata(sds_id_ash(6), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
     
     istatus = sfwdata(sds_id_ash(7), sds_start, sds_stride, sds_edge, &
                       ash_qf(:, j_min:sds_edge(2) + j_min - 1)) + istatus
     
     IF (ash_file_flag == 2) THEN
     
       call SCALE_VECTOR_I2_RANK2(ash_emiss,sym%LINEAR_SCALING,min_ec,max_ec,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(8), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
		       
       call SCALE_VECTOR_I2_RANK2(ash_emiss_tot,sym%LINEAR_SCALING,min_ec,max_ec,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(9), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
		       
       call SCALE_VECTOR_I2_RANK2(ash_beta1112,sym%LINEAR_SCALING,min_beta,max_beta,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(10), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
		       
       call SCALE_VECTOR_I2_RANK2(ash_beta1112_tot,sym%LINEAR_SCALING,min_beta,max_beta,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(11), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
		       
       call SCALE_VECTOR_I2_RANK2(ash_beta1112_tot_multi_low,sym%LINEAR_SCALING,min_beta,max_beta,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(12), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
		       
       call SCALE_VECTOR_I2_RANK2(ash_beta1112_tot_multi_mid,sym%LINEAR_SCALING,min_beta,max_beta,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(13), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
		       
       istatus = sfwdata(sds_id_ash(14), sds_start, sds_stride, sds_edge, &
                         ash_generic1(:, j_min:sds_edge(2) + j_min - 1)) + istatus
		       
       istatus = sfwdata(sds_id_ash(15), sds_start, sds_stride, sds_edge, &
                         ash_generic2(:, j_min:sds_edge(2) + j_min - 1)) + istatus

       call SCALE_VECTOR_I2_RANK2(ash_temperature,sym%LINEAR_SCALING,min_tc,max_tc,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(16), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
		       
       call SCALE_VECTOR_I2_RANK2(ash_vis_optical_depth,sym%LINEAR_SCALING,0.0,15.0,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(17), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
                       
       call SCALE_VECTOR_I2_RANK2(ash_pressure,sym%LINEAR_SCALING,0.0,15.0,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(18), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus                       

       call SCALE_VECTOR_I2_RANK2(ash_beta1112_opaque,sym%LINEAR_SCALING,min_beta,max_beta,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(19), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
     
       call SCALE_VECTOR_I2_RANK2(ash_emissivity_error,sym%LINEAR_SCALING,min_ec_std,max_ec_std,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(20), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
                       
       call SCALE_VECTOR_I2_RANK2(ash_temperature_error,sym%LINEAR_SCALING,min_tc_std,max_tc_std,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(21), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
                       
       call SCALE_VECTOR_I2_RANK2(ash_beta1112_error,sym%LINEAR_SCALING,min_beta_std,max_beta_std,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(22), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
       
       call SCALE_VECTOR_I2_RANK2(ash_probability,sym%LINEAR_SCALING,0.0,1.0,missing_value_real4,two_byte_temp)
       istatus = sfwdata(sds_id_ash(23), sds_start, sds_stride, sds_edge, &
                         two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
                         
     ENDIF
     
     !--- check for and report errors
     if (istatus /= 0) then
        print *, EXE_PROMPT, MOD_PROMPT, "Error writing to ash file: ", istatus
     endif

 endif

!-------------------------------------------------------------------------
! write to level2 file
!-------------------------------------------------------------------------
   if (level2_file_flag == sym%YES) then

      istatus = 0

      !--- scan number
      istatus = sfwdata(sds_id_level2(sds_num_level2_scanline), sds_start(2), sds_stride(2),          &
                        sds_edge(2), scan_number(j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- scan time
      istatus = sfwdata(sds_id_level2(sds_num_level2_time), sds_start(2), sds_stride(2), sds_edge(2),  &
                        utc_scan_time_hours(j_min:sds_edge(2) + j_min - 1)) + istatus
 
      !--- bad scan flag
      istatus = sfwdata(sds_id_level2(sds_num_level2_bad_scan), sds_start(2), sds_stride(2), sds_edge(2),  &
                       bad_scan_flag(j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- ascending/descending flag
      istatus = sfwdata(sds_id_level2(sds_num_level2_asc_flag), sds_start(2), sds_stride(2), sds_edge(2),  &
                ascend(j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- packed pixel metadata
      one_byte_temp = 0
      one_byte_temp = ishft(ch3a_on_pixel_mask,3)+ &
                      ishft(solar_contamination_mask,1) + bad_pixel_mask
      istatus = sfwdata(sds_id_level2(sds_num_level2_meta_data), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- latitude anchors
      call SCALE_VECTOR_I2_RANK2(lat_anchor_1b,sym%LINEAR_SCALING,min_lat,max_lat,missing_value_real4,two_byte_anchor_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_lat),sds_start,sds_stride,anchor_sds_edge, &
                   two_byte_anchor_temp(:, j_min:anchor_sds_edge(2) + j_min - 1)) + istatus

      !--- longitude anchors
      call SCALE_VECTOR_I2_RANK2(lon_anchor_1b,sym%LINEAR_SCALING,min_lon,max_lon,missing_value_real4,two_byte_anchor_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_lon),sds_start,sds_stride,anchor_sds_edge,  &
                   two_byte_anchor_temp(:, j_min:anchor_sds_edge(2) + j_min - 1)) + istatus

      !--- sensor zenith anchors
      call SCALE_VECTOR_I1_RANK2(satzen_anchor,sym%LINEAR_SCALING,min_ang,max_ang,missing_value_real4,one_byte_anchor_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_zen),sds_start,sds_stride,anchor_sds_edge,  &
                        one_byte_anchor_temp(:, j_min:anchor_sds_edge(2) + j_min - 1)) + istatus

      !--- solar zenith anchors
      call SCALE_VECTOR_I1_RANK2(solzen_anchor,sym%LINEAR_SCALING,min_ang,max_ang,missing_value_real4,one_byte_anchor_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_solzen),sds_start,sds_stride,anchor_sds_edge,  &
                        one_byte_anchor_temp(:, j_min:anchor_sds_edge(2) + j_min - 1)) + istatus

      !--- relative azimuth anchors
      call SCALE_VECTOR_I1_RANK2(relaz_anchor,sym%LINEAR_SCALING,min_ang,max_ang,missing_value_real4,one_byte_anchor_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_relaz),sds_start,sds_stride,anchor_sds_edge,  &
                        one_byte_anchor_temp(:, j_min:anchor_sds_edge(2) + j_min - 1)) + istatus

      !--- solar azimuth anchors
      call SCALE_VECTOR_I1_RANK2(solaz_anchor,sym%LINEAR_SCALING,min_solaz,max_solaz,missing_value_real4,one_byte_anchor_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_solaz),sds_start,sds_stride,anchor_sds_edge,  &
                        one_byte_anchor_temp(:, j_min:anchor_sds_edge(2) + j_min - 1)) + istatus

      !--- packed land cover (land,snow,coast masks)
      one_byte_temp = 0
      one_byte_temp = ishft(land,5) + ishft(snow,3) + coast_mask
      istatus = sfwdata(sds_id_level2(sds_num_level2_land), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
!--- observations

      !-- ch1 reflectance
      call SCALE_VECTOR_I2_RANK2(alb1,sym%LINEAR_SCALING,min_alb1,max_alb1,missing_value_real4,two_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ch1), sds_start, sds_stride, sds_edge, &
                        two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !-- ch2 reflectance
      call SCALE_VECTOR_I2_RANK2(alb2,sym%LINEAR_SCALING,min_alb1,max_alb1,missing_value_real4,two_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ch2), sds_start, sds_stride, sds_edge, &
                        two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !-- ch3a reflectance
      call SCALE_VECTOR_I1_RANK2(alb3a,sym%LINEAR_SCALING,min_alb3a,max_alb3a,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ch3a), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !-- ch3b reflectance
      call SCALE_VECTOR_I1_RANK2(alb3b,sym%LINEAR_SCALING,min_alb3b,max_alb3b,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ch3b), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- ch3b - ch4 temperature
      call SCALE_VECTOR_I1_RANK2(bt3b4,sym%LINEAR_SCALING,min_btd_3b_4,max_btd_3b_4,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ch3b4), sds_start, sds_stride, sds_edge, &
                       one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- ch4 temperature
      call SCALE_VECTOR_I2_RANK2(bt4,sym%LINEAR_SCALING,min_t4,max_t4,missing_value_real4,two_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ch4), sds_start, sds_stride, sds_edge, &
                       two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- ch4 - ch5 temperature
      call SCALE_VECTOR_I1_RANK2(bt45,sym%LINEAR_SCALING,min_t45,max_t45,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ch45), sds_start, sds_stride, sds_edge, &
                       one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

     !--- ch1_std_3x3
     call SCALE_VECTOR_I1_RANK2(alb1_std_3x3,sym%LINEAR_SCALING,min_alb1_std,max_alb1_std,missing_value_real4,one_byte_temp)
     istatus = sfwdata(sds_id_level2(sds_num_level2_ch1_std), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

     !--- bt4_std_3x3
     call SCALE_VECTOR_I1_RANK2(bt4_std_3x3,sym%LINEAR_SCALING,min_t4_std,max_t4_std,missing_value_real4,one_byte_temp)
     istatus = sfwdata(sds_id_level2(sds_num_level2_ch4_std), sds_start, sds_stride, sds_edge, &
                     one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!-------- cloud properties

      !--- cloud probability
      call SCALE_VECTOR_I1_RANK2(posterior_cld_probability, &
                                 sym%LINEAR_SCALING,min_frac,max_frac,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_cldprob), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus



if (ncdc_level2_flag == sym%NO) then

      !--- cld type
      istatus = sfwdata(sds_id_level2(sds_num_level2_ctype), sds_start,sds_stride,sds_edge,     &
                        cld_type(:,j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- cld pressure
      call SCALE_VECTOR_I1_RANK2(Pc_sw,sym%LINEAR_SCALING,min_pc,max_pc,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ctp), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- cld temperature
      call SCALE_VECTOR_I1_RANK2(Tc_sw,sym%LINEAR_SCALING,min_tc,max_tc,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ctt), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- cld emissivity from split window
      call SCALE_VECTOR_I1_RANK2(ec_sw,sym%LINEAR_SCALING,min_ec,max_ec,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ec), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- beta from split window
      call SCALE_VECTOR_I1_RANK2(beta_sw,sym%LINEAR_SCALING,min_beta,max_beta,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_beta), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- cld optical depth from vis
      call SCALE_VECTOR_I2_RANK2(tau_vis,sym%LINEAR_SCALING,min_tau,max_tau,missing_value_real4,two_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_cod), sds_start, sds_stride, sds_edge, &
                        two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- cld particle size from vis
      call SCALE_VECTOR_I2_RANK2(re_vis,sym%LINEAR_SCALING,min_re,max_re,missing_value_real4,two_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ceps), sds_start, sds_stride, sds_edge, &
                        two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- cloud albedo
      call SCALE_VECTOR_I1_RANK2(cloud_063um_albedo, &
                                 sym%LINEAR_SCALING,min_albedo,max_albedo,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_cldalb), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- cloud transmission
      call SCALE_VECTOR_I1_RANK2(cloud_063um_transmission_solar, &
                                 sym%LINEAR_SCALING,min_transmission,max_transmission,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_cldtrn), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- cloud fraction
      call SCALE_VECTOR_I1_RANK2(cloud_fraction_3x3, &
                                 sym%LINEAR_SCALING,min_frac,max_frac,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_cldfrac), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- cloud fraction uncertainty
      call SCALE_VECTOR_I1_RANK2(cloud_fraction_uncer_3x3, &
                                 sym%LINEAR_SCALING,min_frac,max_frac,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_cldfracuncer), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- non-cloud props

      !--- etrop
      call SCALE_VECTOR_I1_RANK2(emiss_11_tropo_rtm,sym%LINEAR_SCALING,min_etropo,max_etropo,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_etrop), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- aot1
      call SCALE_VECTOR_I2_RANK2(aot1,sym%LINEAR_SCALING,min_aot,max_aot,missing_value_real4,two_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_aot1), sds_start, sds_stride, sds_edge, &
                        two_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- olr
      call SCALE_VECTOR_I1_RANK2(olr, &
                                 sym%LINEAR_SCALING,min_olr,max_olr,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_olr), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- ndvi surface corrected
      call SCALE_VECTOR_I1_RANK2(ndvi_sfc,sym%LINEAR_SCALING,min_ndvi,max_ndvi,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_ndvi), sds_start, sds_stride, sds_edge, &
                       one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- surface temperature
      call SCALE_VECTOR_I1_RANK2(tsfc,sym%LINEAR_SCALING,min_tsfc,max_tsfc,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_tsfc), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- remote sensing reflectance
      call SCALE_VECTOR_I1_RANK2(rsr,sym%LINEAR_SCALING,min_rsr,max_rsr,missing_value_real4,one_byte_temp)
      istatus = sfwdata(sds_id_level2(sds_num_level2_rsr), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

!--- quality flags

      !--- first level 2 packed quality flags
      one_byte_temp = 0
      one_byte_temp = ishft(tc_sw_qf,6) + ishft(ec_sw_qf,4) + ishft(beta_sw_qf,2) + tau_vis_qf
      istatus = sfwdata(sds_id_level2(sds_num_level2_qf1), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus

      !--- second level 2 packed quality flags
      one_byte_temp = 0
      one_byte_temp = ishft(re_vis_qf,6) + ishft(aot_qf,4) + ishft(rsr_qf,2) + cld_mask

      istatus = sfwdata(sds_id_level2(sds_num_level2_qf2), sds_start, sds_stride, sds_edge, &
                        one_byte_temp(:, j_min:sds_edge(2) + j_min - 1)) + istatus
 
endif  !end of ncdc_level2_flag


     !--- check for and report errors
     if (istatus /= 0) then
        print *, EXE_PROMPT, MOD_PROMPT, "Error writing to level2 file: ", istatus
     endif

    endif


end subroutine WRITE_PIXEL_HDF_RECORDS


!====================================================================
! SUBROUTINE Name: CLOSE_PIXEL_HDF_FILES
!
! Function:
!   Closes appropriate Level 2 files
!
! Description:
!   This subroutine, given the flags that determine which files have 
!   been created, closes the open output files.
!
!====================================================================
subroutine CLOSE_PIXEL_HDF_FILES(cmr_file_flag, &
                                 nav_file_flag, &
                                 sst_file_flag, &
                                 cld_file_flag, &
                                 obs_file_flag, &
                                 geo_file_flag, &
                                 rtm_file_flag, &
                                 ash_file_flag, &
                                 level2_file_flag)

 integer, intent(in):: cmr_file_flag
 integer, intent(in):: nav_file_flag
 integer, intent(in):: obs_file_flag
 integer, intent(in):: geo_file_flag
 integer, intent(in):: rtm_file_flag
 integer, intent(in):: ash_file_flag
 integer, intent(in):: level2_file_flag
 integer, intent(in):: sst_file_flag
 integer, intent(in):: cld_file_flag

 integer:: isds
 integer:: istatus
 integer:: num_ash_sds

! HDF function declarations
 integer:: sfsnatt
 integer:: sfendacc
 integer:: sfend


!-----------------------------------------------------------------------
!--- close cld file
!------------------------------------------------------------------------
  if (cld_file_flag == sym%YES) then
    istatus = 0
    istatus = sfsnatt(sd_id_cld, "NUMBER_OF_ELEMENTS", DFNT_INT32,1,num_pix)+istatus
    istatus = sfsnatt(sd_id_cld, "NUMBER_OF_SCANS_LEVEL1B", DFNT_INT32,1,num_scans)+istatus
    istatus = sfsnatt(sd_id_cld, "NUMBER_OF_SCANS_LEVEL2", DFNT_INT32,1,num_scans_hdf)+istatus
    do isds = 1, num_cld_sds
     istatus = sfendacc(sds_id_cld(isds)) + istatus
    enddo
    istatus = sfend(sd_id_cld) + istatus
    if (istatus /= 0) then
      print *, EXE_PROMPT, MOD_PROMPT, "Error closing cld hdf file"
    endif
  endif
                                                                                                                                            
!-----------------------------------------------------------------------
!--- close sst file
!------------------------------------------------------------------------
  if (sst_file_flag == sym%YES) then
    istatus = 0
    istatus = sfsnatt(sd_id_sst, "NUMBER_OF_ELEMENTS", DFNT_INT32,1,num_pix)+istatus
    istatus = sfsnatt(sd_id_sst, "NUMBER_OF_SCANS_LEVEL1B", DFNT_INT32,1,num_scans)+istatus
    istatus = sfsnatt(sd_id_sst, "NUMBER_OF_SCANS_LEVEL2", DFNT_INT32,1,num_scans_hdf)+istatus
    do isds = 1, num_sst_sds
     istatus = sfendacc(sds_id_sst(isds)) + istatus
    enddo
    istatus = sfend(sd_id_sst) + istatus
    if (istatus /= 0) then
      print *, EXE_PROMPT, MOD_PROMPT, "Error closing sst hdf file"
    endif
 endif

!------------------------------------------------------------------------
!--- close cmr file
!------------------------------------------------------------------------
  if (cmr_file_flag == sym%YES)  then
    istatus = 0
    istatus = sfsnatt(sd_id_cmr, "NUMBER_OF_ELEMENTS", DFNT_INT32,1,num_pix)+istatus
    istatus = sfsnatt(sd_id_cmr, "NUMBER_OF_SCANS_LEVEL1B", DFNT_INT32,1,num_scans)+istatus
    istatus = sfsnatt(sd_id_cmr, "NUMBER_OF_SCANS_LEVEL2", DFNT_INT32,1,num_scans_hdf)+istatus
    do isds = 1, num_cmr_sds
     istatus = sfendacc(sds_id_cmr(isds)) + istatus
    enddo
    istatus = sfend(sd_id_cmr) + istatus
    if (istatus /= 0) then
      print *, EXE_PROMPT, MOD_PROMPT, "Error closing cmr hdf file"
    endif
 endif

!------------------------------------------------------------------------
!--- close nav file
!------------------------------------------------------------------------
  if (nav_file_flag == sym%YES) then
    istatus = 0
    istatus = sfsnatt(sd_id_nav, "NUMBER_OF_ELEMENTS", DFNT_INT32,1,num_pix)+istatus
    istatus = sfsnatt(sd_id_nav, "NUMBER_OF_SCANS_LEVEL1B", DFNT_INT32,1,num_scans)+istatus
    istatus = sfsnatt(sd_id_nav, "NUMBER_OF_SCANS_LEVEL2", DFNT_INT32,1,num_scans_hdf)+istatus
    do isds = 1, num_nav_sds
     istatus = sfendacc(sds_id_nav(isds)) + istatus
    enddo
    istatus = sfend(sd_id_nav) + istatus
    if (istatus /= 0) then
      print *, EXE_PROMPT, MOD_PROMPT, "Error closing nav hdf file"
    endif
 endif

!------------------------------------------------------------------------
!--- close obs file
!------------------------------------------------------------------------
  if (obs_file_flag == sym%YES) then
    istatus = 0
    istatus = sfsnatt(sd_id_obs, "NUMBER_OF_ELEMENTS", DFNT_INT32,1,num_pix)+istatus
    istatus = sfsnatt(sd_id_obs, "NUMBER_OF_SCANS_LEVEL1B", DFNT_INT32,1,num_scans)+istatus
    istatus = sfsnatt(sd_id_obs, "NUMBER_OF_SCANS_LEVEL2", DFNT_INT32,1,num_scans_hdf)+istatus
    do isds = 1, num_obs_sds
     istatus = sfendacc(sds_id_obs(isds)) + istatus
    enddo
    istatus = sfend(sd_id_obs) + istatus
    if (istatus /= 0) then
      print *, EXE_PROMPT, MOD_PROMPT, "Error closing obs hdf file"
    endif
  endif

!------------------------------------------------------------------------
!--- close geo file
!------------------------------------------------------------------------
  if (geo_file_flag == sym%YES) then
    istatus = 0
    istatus = sfsnatt(sd_id_geo, "NUMBER_OF_ELEMENTS", DFNT_INT32,1,num_pix)+istatus
    istatus = sfsnatt(sd_id_geo, "NUMBER_OF_SCANS_LEVEL1B", DFNT_INT32,1,num_scans)+istatus
    istatus = sfsnatt(sd_id_geo, "NUMBER_OF_SCANS_LEVEL2", DFNT_INT32,1,num_scans_hdf)+istatus
    do isds = 1, num_geo_sds
     istatus = sfendacc(sds_id_geo(isds)) + istatus
    enddo
    istatus = sfend(sd_id_geo) + istatus
    if (istatus /= 0) then
      print *, EXE_PROMPT, MOD_PROMPT, "Error closing geo hdf file"
    endif
  endif

!------------------------------------------------------------------------
!--- close rtm file
!------------------------------------------------------------------------
  if (rtm_file_flag == sym%YES) then
    istatus = 0
    istatus = sfsnatt(sd_id_rtm, "NUMBER_OF_ELEMENTS", DFNT_INT32,1,num_pix)+istatus
    istatus = sfsnatt(sd_id_rtm, "NUMBER_OF_SCANS_LEVEL1B", DFNT_INT32,1,num_scans)+istatus
    istatus = sfsnatt(sd_id_rtm, "NUMBER_OF_SCANS_LEVEL2", DFNT_INT32,1,num_scans_hdf)+istatus
    do isds = 1, num_rtm_sds
     istatus = sfendacc(sds_id_rtm(isds)) + istatus
    enddo
    istatus = sfend(sd_id_rtm) + istatus
    if (istatus /= 0) then
      print *, EXE_PROMPT, MOD_PROMPT, "Error closing rtm hdf file"
    endif
  endif

!------------------------------------------------------------------------
!--- close level2 file
!------------------------------------------------------------------------
  istatus = 0
  istatus = sfsnatt(sd_id_level2, "NUMBER_OF_ELEMENTS", DFNT_INT32,1,num_pix)+istatus
  istatus = sfsnatt(sd_id_level2, "NUMBER_OF_ANCHORS", DFNT_INT32,1,num_anchors)+istatus
  istatus = sfsnatt(sd_id_level2, "NUMBER_OF_SCANS_LEVEL1B", DFNT_INT32,1,num_scans)+istatus
  istatus = sfsnatt(sd_id_level2, "NUMBER_OF_SCANS_LEVEL2", DFNT_INT32,1,num_scans_hdf)+istatus
  if (level2_file_flag == sym%YES) then
    do isds = 1, num_sds_ncdc_level2
     istatus = sfendacc(sds_id_level2(isds)) + istatus
    enddo
    if (ncdc_level2_flag == sym%NO) then
      do isds = num_sds_ncdc_level2 +1, num_level2_sds
       istatus = sfendacc(sds_id_level2(isds)) + istatus
      enddo
    endif
    istatus = sfend(sd_id_level2) + istatus
    if (istatus /= 0) then
      print *, EXE_PROMPT, MOD_PROMPT, "Error closing level2 hdf file"
    endif
  endif

!------------------------------------------------------------------------
!--- close ash file
!------------------------------------------------------------------------
  istatus = 0
  istatus = sfsnatt(sd_id_ash, "NUMBER_OF_ELEMENTS", DFNT_INT32,1,num_pix)+istatus
  istatus = sfsnatt(sd_id_ash, "NUMBER_OF_SCANS_LEVEL1B", DFNT_INT32,1,num_scans)+istatus
  istatus = sfsnatt(sd_id_ash, "NUMBER_OF_SCANS_LEVEL2", DFNT_INT32,1,num_scans_hdf)+istatus
  
  if (ash_file_flag > 0) then
    
    num_ash_sds = num_ash_sds_oper
    if (ash_file_flag == 2) num_ash_sds = num_ash_sds_diag
    
    do isds = 1, num_ash_sds
     istatus = sfendacc(sds_id_ash(isds)) + istatus
    enddo
    istatus = sfend(sd_id_ash) + istatus
    if (istatus /= 0) then
      print *, EXE_PROMPT, MOD_PROMPT, "Error closing ash hdf file"
    endif
    
  endif

end subroutine CLOSE_PIXEL_HDF_FILES

!====================================================================
! SUBROUTINE Name: DEFINE_PIXEL_2D_SDS
!
! Function:
!   Defines a 2D SDS for the level 2 files
!
! Description:
!   This subroutine, given the inputs, creates the SDSs inside the various
!   files. 
!
!====================================================================
  subroutine DEFINE_PIXEL_2D_SDS(sds_id,     &
                                 sd_id,      &
                                 sds_dims,   &
                                 sds_chunk,  &
                                 sds_name,   &
                                 sds_standard_name,   &
                                 sds_long_name, &
                                 sds_type,   &
                                 scaled,     &
                                 sds_min,    &
                                 sds_max,    &
                                 sds_units,  &
                                 sds_missing,&
                                 istatus)

    integer, intent(out):: sds_id
    integer, intent(in):: sd_id
    integer, dimension(2), intent(in):: sds_dims
    integer, dimension(2), intent(in):: sds_chunk
    integer, intent(in):: sds_type
    real, intent(in):: sds_min
    real, intent(in):: sds_max
    real, intent(in):: sds_missing
    integer(kind=int1), intent(in):: scaled
    character(len=*), intent(in):: sds_name
    character(len=*), intent(in):: sds_standard_name
    character(len=*), intent(in):: sds_long_name
    character(len=*), intent(in):: sds_units
    integer, intent(out):: istatus
    integer:: dim_id
    integer(kind=int4):: scaled_min
    integer(kind=int4):: scaled_max
    integer(kind=int4):: scaled_missing
    real(kind=real4):: add_offset
    real(kind=real4):: scale_factor

    integer:: sfcreate
    integer:: sfsnatt
    integer:: sfscatt
    integer:: sfdimid
    integer:: sfsdmname
    integer:: sfschnk

     istatus = 0 

     sds_id = sfcreate(sd_id,sds_name,sds_type,sds_rank,sds_dims)
     istatus = sfsnatt(sds_id, "SCALED", DFNT_INT8, 1, scaled) + istatus
     istatus = sfscatt(sds_id, "UNITS", DFNT_CHAR8, len_trim(sds_units), trim(sds_units)) + istatus
     istatus = sfscatt(sds_id, "STANDARD_NAME", DFNT_CHAR8, len_trim(sds_standard_name),  &
                                trim(sds_standard_name)) + istatus
     istatus = sfscatt(sds_id, "LONG_NAME", DFNT_CHAR8, len_trim(sds_long_name),  &
                                trim(sds_long_name)) + istatus

     dim_id = sfdimid(sds_id, 0)
     if (sds_dims(1) == num_anchors) then
        istatus = sfsdmname(dim_id,"anchors_across_track") + istatus
     else
        istatus = sfsdmname(dim_id,"pixels_across_track") + istatus
     endif

     dim_id = sfdimid(sds_id, 1)
     istatus = sfsdmname(dim_id,"scan_lines_along_track") + istatus

     if (subset_pixel_hdf_flag == sym%NO) then      !compression not compatible
        istatus = sfschnk(sds_id,sds_chunk,comp_type,comp_prm)+istatus
     endif
     istatus = sfsnatt(sds_id, "RANGE_MISSING", DFNT_FLOAT32, 1, sds_missing) + istatus

     !--- determined if a scaled sds, if so write needed attributes for scaling
     if (scaled > 0) then

       !--- determine scaled ranges based on sds_type
       if (sds_type == DFNT_INT8) then
           scaled_min = one_byte_min
           scaled_max = one_byte_max
           scaled_missing = missing_value_int1
       elseif (sds_type == DFNT_INT16) then
           scaled_min = two_byte_min
           scaled_max = two_byte_max
           scaled_missing = missing_value_int2
       endif

      !--- determine offset and scale factor
      scale_factor = (sds_max - sds_min) / (scaled_max - scaled_min)
      add_offset =  sds_min - scale_factor*scaled_min

      !--- write remaining attributes
      istatus = sfsnatt(sds_id, "RANGE_MIN", DFNT_FLOAT32, 1, sds_min) + istatus
      istatus = sfsnatt(sds_id, "RANGE_MAX", DFNT_FLOAT32, 1, sds_max) + istatus
      istatus = sfsnatt(sds_id, "SCALED_MIN", DFNT_INT32, 1, scaled_min) + istatus
      istatus = sfsnatt(sds_id, "SCALED_MAX", DFNT_INT32, 1, scaled_max) + istatus
      istatus = sfsnatt(sds_id, "SCALED_MISSING", DFNT_INT32, 1, scaled_missing) + istatus
      istatus = sfsnatt(sds_id, "_FILLVALUE", DFNT_INT32, 1, scaled_missing) + istatus
      istatus = sfsnatt(sds_id, "SCALE_FACTOR", DFNT_FLOAT32, 1, scale_factor) + istatus
      istatus = sfsnatt(sds_id, "ADD_OFFSET", DFNT_FLOAT32, 1, add_offset) + istatus

   endif

  end subroutine DEFINE_PIXEL_2D_SDS

end module LEVEL2_ROUTINES
