! $Id: subset_mapped.f90,v 1.3 2010/09/23 21:21:38 wstraka Exp $
!----------------------------------------------------------------------------
!@***h* PATMOSX/subset_mapped.f90
! 
! NAME
!   subset_mapped.f90
! 
! LOCATION
!   /patmosx/src/subset_mapped.f90
! 
! PURPOSE
!  This Module contains the subroutines needed to subset the HD4 files
! 
! DESCRIPTION
!  This Module contains the subroutines needed to subset the HD4 files
!
! AUTHOR
!   Andrew Heidinger, NOAA/NESDIS
! 
! CREATION DATE
!   Oct 2007
!
! COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
!   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
!
! NOTES
!   none
!
! EXTERNALS
!   READ_CLAVRX_HDF_GLOBAL_ATTRIBUTES
!   WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES
!
!@*****
!------------------------------------------------------------------------
!
! PROGRAM/SUBROUTINE name: SUBSET
! 
! this Module contains the subroutines needed to subset the HD4 files
!
! Author: Andrew Heidinger
! Date October 2007
!----------------------------------------------------------------------
program SUBSET
 use CONSTANTS
 use HDF
 use HDF_PARAMS
 use NUMERICAL_ROUTINES
 use SCALING_PARAMETERS
 implicit none

!----
 character(len=100)::  dir_in, dir_out, file_in, file_out, file_temp
 integer:: ios

 character(len=4):: year_string
 character(len=3):: jday_string,sat_string,node_string,grid_resolution_string
 character(len=1):: grid_format_string
 real(kind=real4):: grid_resolution_input
 integer(kind=int4):: grid_format_flag_input
 integer:: compress_flag

!--- global attributes
 integer(kind=int4):: sd_id_in, sd_id_out
 integer(kind=int4):: num_cells,num_cells_with_data
 integer:: therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal, &
                        sst_anal_opt, cloud_type_ir_flag, nwp_flag
 integer(kind=int4)::  start_time,end_time
 integer(kind=int2)::  start_year,end_year,start_day,end_day
 real(kind=real4)::  grid_resolution, ch1_gain_low, ch1_gain_high, &
       ch2_gain_low, ch2_gain_high, &
       ch3a_gain_low, ch3a_gain_high, &
       ch1_switch_count, ch1_dark_count, &
       ch2_switch_count, ch2_dark_count, &
       ch3a_switch_count, ch3a_dark_count, &
       sun_earth_distance, &
       c1, c2, a_3b, b_3b, nu_3b, &
       a_4, b_4, nu_4, a_5, b_5, nu_5, &
       solar_3b_nu
                                                                                                                                                         
 character(len=100):: file_name, file_1b, creator, machine,  &
                    plang, hdf_ver, hdf_timestamp, data_type,grid_format

 real, dimension(:), allocatable:: sds_data

!--- sds attributes
  character(len=100):: sds_name
  real:: unscaled_min, unscaled_max, unscaled_missing
  integer:: scaled_min, scaled_max, scaled_missing
  integer(kind=int1):: scaled
  character(len=100):: sds_units
  integer:: sds_data_type
  integer:: sds_rank, num_attrs
  integer, dimension(1):: dimsizes

!--- local
  integer:: ifile,n_files, i, istatus, sds_index, sds_id, istatus_write, &
            num_datasets, num_global_attrs, iupdate,grid_format_flag

!--- hdf
   integer:: sfstart, sfend, sfginfo,sfselect, sffinfo

!--- sds names to include in subset
   integer:: num_sds_subset = 1
   character(len=12), dimension(num_subset_sds):: sds_name_subset 

!===============================================================================
! Begin Executable Code
!===============================================================================

!--- sds names to include in subset
     sds_name_subset(1) = "frac_total_cld"

!---- Find the machine name to use in HDF global attribute
 call GET_MACHINE_NAME()

dir_in = "/data3/patmosx_v4/monthly/"
dir_out = "/data3/patmosx_v4/monthly/"
file_in = "shit.hdf"
file_out = "temp.hdf"

!--- open input file
    sd_id_in = sfstart(trim(dir_in)//trim(file_in), DFACC_READ)
    if (sd_id_in == FAIL) then
      print *, "failed open for read on orbital gridded file, stopping"
      stop
    endif

!--- read global attributes for first file - these will serve as those
!--- for the final composite(except for time)
grid_format=" "
call READ_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_in,data_type,file_name,file_1b, &
                           start_year,end_year,start_day,end_day,start_time,end_time,&
                           num_cells,num_cells_with_data,grid_format,grid_resolution, &
                           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
                           cloud_type_ir_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
                           ch1_switch_count, ch1_dark_count, &
                           ch2_gain_low, ch2_gain_high, &
                           ch2_switch_count, ch2_dark_count, &
                           ch3a_gain_low, ch3a_gain_high, &
                           ch3a_switch_count, ch3a_dark_count, &
                           sun_earth_distance, &
                           c1, c2, a_3b, b_3b, nu_3b, &
                           a_4, b_4, nu_4, a_5, b_5, nu_5, &
                           solar_3b_nu, &
                           creator, plang, hdf_ver,hdf_timestamp,machine)

!--- allocate array for sds data
 allocate(sds_data(num_cells_with_data)

!--- determine number of sds's in these files (assume the same)
     istatus = sffinfo(sd_id_in,num_datasets,num_global_attrs)

!--- open file for writing
    sd_id_out = sfstart(trim(dir_out)//trim(file_out),DFACC_CREATE)

     if (sd_id_out <  0) then
       print *, "composite HDF file creation failed. Exiting...", asc_des_node
       stop 38
     endif

!--- write global attributes to output file
 call WRITE_CLAVRX_HDF_GLOBAL_ATTRIBUTES(sd_id_out,"GRIDCELL",trim(file_out),trim(file_1b), &
           start_year,end_year,start_day,end_day,start_time,end_time,&
           num_cells,num_cells_with_data,grid_format,grid_resolution, &
           therm_cal_1b,ref_cal_1b,nav_flag,use_sst_anal,sst_anal_opt, &
           cloud_type_ir_flag, nwp_flag, ch1_gain_low, ch1_gain_high, &
           ch1_switch_count, ch1_dark_count, &
           ch2_gain_low, ch2_gain_high, &
           ch2_switch_count, ch2_dark_count, &
           ch3a_gain_low, ch3a_gain_high, &
           ch3a_switch_count, ch3a_dark_count, &
           sun_earth_distance, &
           c1, c2, a_3b, b_3b, nu_3b, &
           a_4, b_4, nu_4, a_5, b_5, nu_5, &
           solar_3b_nu)

!--- loop through each parameter 
   istatus = 0
   sds_index_loop: do sds_index = 0,num_datasets-1

!--- get sds index number
         sds_id = sfselect(sd_id_in,sds_index)

!-- read in sds information
         sds_name = " "
         istatus = sfginfo(sds_id, sds_name, sds_rank, dimsizes, sds_data_type, num_attrs) + istatus

         if (istatus /= 0) then
            print *, "could not find sds ", sds_index
            exit
         endif

!--- see if sds_name is in the desired list
    do i = 1, num_sds_subset
      include_sds_mask(i) =  0
      if (index(sds_name. sds_name_subset(i)) > 0) then
          include_sds_mask(i) = 1
      endif
    end do 

!--- if it is to included in subset, read it in and write it out to output file
    if (include_sds_mask(i)  == 1) then

!--- read sds from input file
         call READ_CLAVRX_HDF4_SDS_RANK1(sd_id_in,num_cells_with_data, &
                              trim(sds_name),sds_data, &
                              sds_data_type,scaled,sds_units, &
                              unscaled_min,unscaled_max,unscaled_missing,&
                              scaled_min,scaled_max,scaled_missing,&
                              istatus)

!--- write out sds to output file
         call WRITE_CLAVRX_HDF4_SDS(sd_id_out,real(sds_data,kind=real4), &
                  trim(sds_name),&
                  sds_data_type,scaled,unscaled_min,unscaled_max,trim(sds_units), unscaled_missing, &
                  "grid_cell_number", compress_flag, istatus_write)

     endif  

   end do sds_index_loop 

!--- close file
   istatus = sfend(sd_id_in)
   istatus = sfend(sd_id_out)


!--- deallocate sds data array
deallocate(sds_data)

end program SUBSET
