c$Id: tranmavhrr.f,v 1.2 2009/02/25 21:33:17 heidinger Exp $
	subroutine tranmavhrr(ancil_data_path,temp,wvmr,ozmr,zena, 
     &                        craft,kban,taut,*)
c@***s* PATMOSX/tranmavhrr.f
c* 
c* NAME
c*   tranmavhrr.f
c* 
c* LOCATION
c*   /patmosx/src/tranmavhrr.f
c* 
c* PURPOSE
c*   Calculate AVHRR transmittances, version of 01.08.06
c*   for TIROS-N, NOAAA-6 ... NOAA-18, METOP-A, ff
c* 
c* DESCRIPTION
c*   Calculate AVHRR transmittances, version of 01.08.06
c*   for TIROS-N, NOAAA-6 ... NOAA-18, METOP-A, ff
c*   LarrabeeStrow/HalWoolf/PaulVanDelst regression model based on
c*   LBLRTM line-by-line transmittances.
c*   Input temperatures, and water-vapor and ozone mixing ratios, must
c*   be defined at the pressure levels in array 'pstd'
c*   (see block data 'reference_atmosphere').
c*
c* AUTHOR
c*   Andrew Heidinger, NOAA/NESDIS
c* 
c* CREATION DATE
c*   unk
c*
c* COPYRIGHT
c*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
c*   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE 
c*   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
c*   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
c*   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
c*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
c*   TO USERS.
c*
c* NOTES
c*   Units: temperature, deg-K; water vapor, g/kg; ozone, ppmv.
c*   Logical units 71-75 are used for coefficient files.
c*   Component tau's are returned through common, product in 'taut'.
c*
c* INPUTS_OUTPUTS
c*   Inputs:
c*     temp  = temperature profile (degK)
c*     wvmr  = water-vapor mixing-ratio profile (g/kg)
c*     ozmr  = ozone mixing-ratio profile (ppmv)
c*     zena  = local zenith angle in degrees
c*     craft = spacecraft: tirosn,noaa06...noaa18, metopa, ... upper or lower case
c*     kban  = band number: 3...5
c*
c*   Outputs:
c*     taut  = total transmittance
c*        *  = alternate return if any coefficient-file I/O problems
c*             in common/taudwo/
c*     taud  = transmittance due to uniformly mixed gases
c*     tauw  = transmittance due to water vapor
c*     tauo  = transmittance due to ozone
c*
c@*****
c*------------------------------------------------------------------------
c * Calculate AVHRR transmittances
c .... version of 01.08.06
c ++++ for TIROS-N, NOAAA-6 ... NOAA-18, METOP-A, ff.

c * LarrabeeStrow/HalWoolf/PaulVanDelst regression model based on
c *	LBLRTM line-by-line transmittances.
c * Input temperatures, and water-vapor and ozone mixing ratios, must
c *	be defined at the pressure levels in array 'pstd'
c *	(see block data 'reference_atmosphere').
c * Units: temperature, deg-K; water vapor, g/kg; ozone, ppmv.
c * Logical units 71-75 are used for coefficient files.
c * Component tau's are returned through common, product in 'taut'.
 
c * Inputs:
c	 temp = temperature profile (degK)
c	 wvmr = water-vapor mixing-ratio profile (g/kg)
c	 ozmr = ozone mixing-ratio profile (ppmv)
c	 zena = local zenith angle in degrees
c	craft = spacecraft: tirosn,noaa06...noaa18, metopa, ... upper or lower case
c	 kban = band number: 3...5

c * Outputs:
c		taut = total transmittance
c		   * = alternate return if any coefficient-file I/O problems
c          in common/taudwo/
c		taud = transmittance due to uniformly mixed gases
c		tauw = transmittance due to water vapor
c		tauo = transmittance due to ozone

	parameter (lfac=4,nk=5,nl=101,nm=nl-1,nr=3)
	parameter (nxc= 4,ncc=nxc+1,lencc=ncc*nm,lenccb=lencc*lfac)
	parameter (nxd= 8,ncd=nxd+1,lencd=ncd*nm,lencdb=lencd*lfac)
	parameter (nxo= 9,nco=nxo+1,lenco=nco*nm,lencob=lenco*lfac)
	parameter (nxl= 2,ncl=nxl+1,lencl=ncl*nm,lenclb=lencl*lfac)
	parameter (nxs=11,ncs=nxs+1,lencs=ncs*nm,lencsb=lencs*lfac)
	parameter (nxw=nxl+nxs)
	common/stdatm/pstd(nl),tstd(nl),wstd(nl),ostd(nl)
	common/taudwo/taud(nl),tauw(nl),tauo(nl)
	dimension temp(*),wvmr(*),ozmr(*),taut(*)
	dimension coefd(ncd,nm,nr),coefo(nco,nm,nr),coefl(ncl,nm,nr)
	dimension coefs(ncs,nm,nr),coefc(ncc,nm,nr),cbufs(lencs)
	dimension pavg(nm),tref(nm),wref(nm),oref(nm)
	dimension tavg(nm),wamt(nm),oamt(nm),secz(nm)
	dimension tauc(nl),tlas(nl),wlas(nl),olas(nl)
	dimension xdry(nxd,nm),xozo(nxo,nm),xwet(nxw,nm),xcon(nxc,nm)
c       character*12 cfile/'avhrncom.dat'/
	character*12 cfile
        character*72 ancil_data_path,pfast_path
c       character*6 craft,clast/'999999'/
	character*6 craft,clast
c       character*3 comp(nk)/'dry','ozo','wco','wtl','wts'/
	character*3 comp(nk)
c       integer*4 iuc(nk),lencf(nk)/lencdb,lencob,lenccb,lenclb,lencsb/
	integer*4 iuc(nk),lencf(nk)
	logical flip(nk),newang,newatm
	data init/0/,tlas/nl*0./,wlas/nl*0./,olas/nl*0./,zlas/-999./
	secant(z)=1./cos(0.01745329*z)

        cfile = "avhrncom.dat"
	clast="999999"
        pfast_path = trim(ancil_data_path)//"pfast/"
        comp = (/'dry','ozo','wco','wtl','wts'/)
        lencf = (/lencdb,lencob,lenccb,lenclb,lencsb/)

	if(craft.ne.clast) then
	   iux=70
	   do l=1,nk
	      cfile(6:8)=comp(l)
	      iux=iux+1
	      iuc(l)=iux

	      open(iuc(l),file=trim(pfast_path)//cfile,
     +             recl=lencf(l),access='direct',
     +             status='old',err=200)
	   enddo

	   call getnumsc(craft,noff)
	   koff=noff*(nr+1)+1
	   krec=koff
c * determine if coefficients need to be byte-flipped
	   do k=1,nk
	      lenk=lencf(k)/lfac
	      read(iuc(k),rec=krec) (cbufs(j),j=1,lenk)
	      flip(k)=cbufs(1).lt.1.0
 	   enddo
c * read in coefficients
	   do k=1,nr
	      krec=k+koff
	      read(iuc(1),rec=krec) ((coefd(i,j,k),i=1,ncd),j=1,nm)
	      read(iuc(2),rec=krec) ((coefo(i,j,k),i=1,nco),j=1,nm)
	      read(iuc(3),rec=krec) ((coefc(i,j,k),i=1,ncc),j=1,nm)
	      read(iuc(4),rec=krec) ((coefl(i,j,k),i=1,ncl),j=1,nm)
	      read(iuc(5),rec=krec) ((coefs(i,j,k),i=1,ncs),j=1,nm)
	   enddo
	   do l=1,nk
	      close(iuc(l))
	   enddo
c * flip if necessary
	   if( flip(1) ) call flip_rtc(coefd,ncd,nm,nr)
	   if( flip(2) ) call flip_rtc(coefo,nco,nm,nr)
	   if( flip(3) ) call flip_rtc(coefc,ncc,nm,nr)
	   if( flip(4) ) call flip_rtc(coefl,ncl,nm,nr)
	   if( flip(5) ) call flip_rtc(coefs,ncs,nm,nr)
	   call conpir(pstd,tstd,wstd,ostd,nl,1,pavg,tref,wref,oref)
	   clast=craft
	endif

	dt=0.
	dw=0.
	do=0.
	do j=1,nl
	   dt=dt+abs(temp(j)-tlas(j))
	   tlas(j)=temp(j)
	   dw=dw+abs(wvmr(j)-wlas(j))
	   wlas(j)=wvmr(j)
	   do=do+abs(ozmr(j)-olas(j))
	   olas(j)=ozmr(j)
	   taud(j)=1.0
	   tauw(j)=1.0
	   tauc(j)=1.0
	   tauo(j)=1.0
	   taut(j)=1.0
	enddo
	datm=dt+dw+do
	newatm=datm.ne.0.
	if(newatm) then
	   call conpir(pstd,temp,wvmr,ozmr,nl,1,pavg,tavg,wamt,oamt)
	endif

	newang=zena.ne.zlas
	if(newang) then
	   zsec=secant(zena)
	   do l=1,nm
	      secz(l)=zsec
	   enddo
	   zlas=zena
	endif

	if(newang.or.newatm) then
	   call calpir(tref,wref,oref,tavg,wamt,oamt,pavg,secz,
     +			 nm,nxd,nxw,nxo,nxc,xdry,xwet,xozo,xcon)
	endif

	k=kban-2
c * dry
	call taudoc(ncd,nxd,nm,coefd(1,1,k),xdry,taud)
c * ozo
	call taudoc(nco,nxo,nm,coefo(1,1,k),xozo,tauo)
c * wet
	call taudoc(ncc,nxc,nm,coefc(1,1,k),xcon,tauc)
	call tauwtr(ncs,ncl,nxs,nxl,nxw,nm,coefs(1,1,k),
     +			coefl(1,1,k),xwet,tauw)
	do j=1,nl
	   tauw(j)=tauw(j)*tauc(j)
	enddo
c * total
	do j=1,nl
	   taut(j)=taud(j)*tauo(j)*tauw(j)
	enddo
	return
200	return1
	end
